@@ -21,7 +21,7 @@ TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: AdvancedMessageDialog.cpp 1177 2006-07-05 11:46:28Z mghie $
+  $Id: AdvancedMessageDialog.cpp 1581 2007-12-05 21:36:54Z babuskov $
 
 */
 
@@ -226,27 +226,28 @@ wxSize TextWrapEngine::computeWrappedExtent(wxDC& dc, const wxString& text,
 {
     int textW = 0, textH = 0;
 
+	const wxChar* const pos = text.c_str();
     const wxChar* p = text.c_str();
     const wxChar* pWrap = 0;
     const wxChar* r = p;
-    while (*p)
+    while (p && *p)
     {
         // scan over non-whitespace
         while (*r > ' ')
             r++;
         int w;
-        dc.GetTextExtent(wxString(p, r), &w, 0);
+        dc.GetTextExtent(text.Mid(p-pos, r-p), &w, 0);
         if (w <= wrapWidth) // partial line fits in wrapWidth
-            pWrap = r; 
+            pWrap = r;
         if (w > wrapWidth || *r == 0)
         {
             int h;
-            dc.GetTextExtent(wxString(p, pWrap), &w, &h);
+            dc.GetTextExtent(text.Mid(p-pos, pWrap-p), &w, &h);
             textW = (w > textW) ? w : textW;
             textH += h;
             p = pWrap;
             // scan over whitespace
-            while (*p != 0 && *p <= ' ')
+            while (p && *p != 0 && *p <= ' ')
                 p++;
             pWrap = 0;
         }
@@ -262,27 +263,28 @@ wxString TextWrapEngine::wrapLine(wxDC& dc, const wxString& text,
 {
     wxString result;
 
+	const wxChar* const pos = text.c_str();
     const wxChar* p = text.c_str();
     const wxChar* pWrap = 0;
     const wxChar* r = p;
-    while (*p)
+    while (p && *p)
     {
         // scan over non-whitespace
         while (*r > ' ')
             r++;
-        wxString partialLine(p, r);
+        wxString partialLine = text.Mid(p-pos, r-p);
         int w;
         dc.GetTextExtent(partialLine, &w, 0);
         if (w <= wrapWidth) // partial line fits in wrapWidth
-            pWrap = r; 
+            pWrap = r;
         if (w > wrapWidth || *r == 0)
         {
             if (!result.empty())
                 result += wxT("\n");
-            result += wxString(p, pWrap);
+            result += text.Mid(p-pos, pWrap-p);
             p = pWrap;
             // scan over whitespace
-            while (*p != 0 && *p <= ' ')
+            while (p && *p != 0 && *p <= ' ')
                 p++;
             pWrap = 0;
         }
@@ -347,7 +349,7 @@ AdvancedMessageDialog::AdvancedMessageDialog(wxWindow* parent, wxArtID iconId,
         false, primaryTextWrapped, 0);
     labelPrimary->SetLabel(primaryTextWrapped);
 
-    TextWrapEngine::computeWordWrap(secondaryText, secondaryLabelFont, 
+    TextWrapEngine::computeWordWrap(secondaryText, secondaryLabelFont,
         wrapWidth, false, secondaryTextWrapped, 0);
     labelSecondary->SetLabel(secondaryTextWrapped);
 
