@@ -1,51 +1,53 @@
 /*
-Copyright (c) 2004, 2005, 2006 The FlameRobin Development Team
+  Copyright (c) 2004-2008 The FlameRobin Development Team
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+  Permission is hereby granted, free of charge, to any person obtaining
+  a copy of this software and associated documentation files (the
+  "Software"), to deal in the Software without restriction, including
+  without limitation the rights to use, copy, modify, merge, publish,
+  distribute, sublicense, and/or sell copies of the Software, and to
+  permit persons to whom the Software is furnished to do so, subject to
+  the following conditions:
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+  The above copyright notice and this permission notice shall be included
+  in all copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: AdvancedSearchFrame.cpp 1189 2006-07-31 18:53:50Z mbabuskov $
+  $Id: AdvancedSearchFrame.cpp 1631 2008-02-02 22:25:30Z mapopa $
 
 */
 //-----------------------------------------------------------------------------
 #include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
-#pragma hdrstop
+    #pragma hdrstop
 #endif
 
 #ifndef WX_PRECOMP
-#include <wx/wx.h>
+    #include <wx/wx.h>
 #endif
 
-#include "dberror.h"
+#include <set>
+
 #include "frutils.h"
-#include "ContextMenuMetadataItemVisitor.h"
+#include "gui/AdvancedSearchFrame.h"
+#include "gui/ContextMenuMetadataItemVisitor.h"
+#include "gui/controls/DBHTreeControl.h"
+#include "gui/MainFrame.h"
+#include "gui/ProgressDialog.h"
+#include "metadata/CreateDDLVisitor.h"
+#include "metadata/database.h"
 #include "metadata/metadataitem.h"
 #include "metadata/root.h"
 #include "metadata/server.h"
-#include "metadata/database.h"
-#include "metadata/CreateDDLVisitor.h"
-#include "ProgressDialog.h"
-#include "MainFrame.h"
-#include "AdvancedSearchFrame.h"
 //-----------------------------------------------------------------------------
 // derived class since we need to catch size event
 class AdjustableListCtrl: public wxListCtrl
@@ -244,6 +246,7 @@ AdvancedSearchFrame::AdvancedSearchFrame(MainFrame* parent)
     SetSizerAndFit(mainSizer);
     Centre();
 
+    // TODO: size(32, 32) missing for SEARCH icon
     #include "search.xpm"
     wxBitmap bmp = wxBitmap(search_xpm);
     wxIcon icon;
@@ -439,7 +442,7 @@ void AdvancedSearchFrame::OnListCtrlResultsItemSelected(wxListEvent& event)
     MainFrame *mf = dynamic_cast<MainFrame *>(GetParent());
     if (mf)
     {
-        myTreeCtrl *tree = mf->getTreeCtrl();
+        DBHTreeControl* tree = mf->getTreeCtrl();
         if (tree)
             tree->selectMetadataItem(m);
     }
@@ -648,7 +651,7 @@ void AdvancedSearchFrame::OnButtonAddDatabaseClick(wxCommandEvent& WXUNUSED(even
 {
     if (choice_database->GetSelection() == 0)   // all connected databases
     {
-        for (int i=1; i<choice_database->GetCount(); ++i)
+        for (int i = 1; i < (int)choice_database->GetCount(); ++i)
         {
             Database *db = (Database *)choice_database->GetClientData(i);
             if (db && db->isConnected())
