@@ -1,27 +1,27 @@
 /*
-Copyright (c) 2004, 2005, 2006 The FlameRobin Development Team
+  Copyright (c) 2004-2008 The FlameRobin Development Team
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+  Permission is hereby granted, free of charge, to any person obtaining
+  a copy of this software and associated documentation files (the
+  "Software"), to deal in the Software without restriction, including
+  without limitation the rights to use, copy, modify, merge, publish,
+  distribute, sublicense, and/or sell copies of the Software, and to
+  permit persons to whom the Software is furnished to do so, subject to
+  the following conditions:
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+  The above copyright notice and this permission notice shall be included
+  in all copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: DatabaseRegistrationDialog.cpp 1163 2006-06-08 11:43:04Z mghie $
+  $Id: DatabaseRegistrationDialog.cpp 1631 2008-02-02 22:25:30Z mapopa $
 
 */
 
@@ -42,6 +42,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <wx/gbsizer.h>
 
 #include "core/StringUtils.h"
+#include "gui/controls/DndTextControls.h"
 #include "gui/DatabaseRegistrationDialog.h"
 #include "gui/StyleGuide.h"
 #include "metadata/database.h"
@@ -61,12 +62,6 @@ DatabaseRegistrationDialog::DatabaseRegistrationDialog(wxWindow* parent,
     setControlsProperties();
     layoutControls();
     updateButtons();
-
-    #include "database32.xpm"
-    wxBitmap bmp = wxBitmap(database32_xpm);
-    wxIcon icon;
-    icon.CopyFromBitmap(bmp);
-    SetIcon(icon);
 }
 //-----------------------------------------------------------------------------
 //! implementation details
@@ -80,11 +75,11 @@ const wxString DatabaseRegistrationDialog::buildName(const wxString& dbPath) con
 void DatabaseRegistrationDialog::createControls()
 {
     label_name = new wxStaticText(getControlsPanel(), -1, _("Display name:"));
-    text_ctrl_name = new wxTextCtrl(getControlsPanel(), ID_textcontrol_name,
-        wxEmptyString);
+    text_ctrl_name = new wxTextCtrl(getControlsPanel(),
+        ID_textcontrol_name, wxEmptyString);
     label_dbpath = new wxStaticText(getControlsPanel(), -1,
         _("Database path:"));
-    text_ctrl_dbpath = new wxTextCtrl(getControlsPanel(),
+    text_ctrl_dbpath = new FileTextControl(getControlsPanel(),
         ID_textcontrol_dbpath, wxEmptyString);
     button_browse = new wxButton(getControlsPanel(), ID_button_browse,
         wxT("..."), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
@@ -104,15 +99,18 @@ void DatabaseRegistrationDialog::createControls()
     label_charset = new wxStaticText(getControlsPanel(), -1, _("Charset:"));
 
     const wxString charset_choices[] = {
-        wxT("NONE"),        wxT("ASCII"),       wxT("BIG_5"),       wxT("CYRL"),        wxT("DOS437"),
-        wxT("DOS737"),      wxT("DOS775"),      wxT("DOS850"),      wxT("DOS852"),      wxT("DOS857"),
-        wxT("DOS858"),      wxT("DOS860"),      wxT("DOS861"),      wxT("DOS862"),      wxT("DOS863"),
-        wxT("DOS864"),      wxT("DOS865"),      wxT("DOS866"),      wxT("DOS869"),      wxT("EUCJ_0208"),
-        wxT("GB_2312"),     wxT("ISO8859_1"),   wxT("ISO8859_2"),   wxT("ISO8859_3"),   wxT("ISO8859_4"),
-        wxT("ISO8859_5"),   wxT("ISO8859_6"),   wxT("ISO8859_7"),   wxT("ISO8859_8"),   wxT("ISO8859_9"),
-        wxT("ISO8859_13"),  wxT("KSC_5601"),    wxT("NEXT"),        wxT("OCTETS"),      wxT("SJIS_0208"),
-        wxT("UNICODE_FSS"), wxT("UTF8"),       wxT("WIN1250"),     wxT("WIN1251"),     wxT("WIN1252"),
-        wxT("WIN1253"),     wxT("WIN1254"),     wxT("WIN1255"),     wxT("WIN1256"),     wxT("WIN1257")
+        wxT("NONE"),        wxT("ASCII"),       wxT("BIG_5"),       wxT("CYRL"),
+        wxT("DOS437"),      wxT("DOS737"),      wxT("DOS775"),      wxT("DOS850"),
+        wxT("DOS852"),      wxT("DOS857"),      wxT("DOS858"),      wxT("DOS860"),
+        wxT("DOS861"),      wxT("DOS862"),      wxT("DOS863"),      wxT("DOS864"),
+        wxT("DOS865"),      wxT("DOS866"),      wxT("DOS869"),      wxT("EUCJ_0208"),
+        wxT("GB_2312"),     wxT("ISO8859_1"),   wxT("ISO8859_2"),   wxT("ISO8859_3"),
+        wxT("ISO8859_4"),   wxT("ISO8859_5"),   wxT("ISO8859_6"),   wxT("ISO8859_7"),
+        wxT("ISO8859_8"),   wxT("ISO8859_9"),   wxT("ISO8859_13"),  wxT("KSC_5601"),
+        wxT("NEXT"),        wxT("OCTETS"),      wxT("SJIS_0208"),   wxT("UNICODE_FSS"),
+        wxT("UTF8"),        wxT("WIN1250"),     wxT("WIN1251"),     wxT("WIN1252"),
+        wxT("WIN1253"),     wxT("WIN1254"),     wxT("WIN1255"),     wxT("WIN1256"),
+        wxT("WIN1257")
     };
 
     combobox_charset = new wxComboBox(getControlsPanel(), -1, wxT("NONE"),
@@ -309,7 +307,12 @@ END_EVENT_TABLE()
 void DatabaseRegistrationDialog::OnBrowseButtonClick(wxCommandEvent& WXUNUSED(event))
 {
     wxString path = ::wxFileSelector(_("Select database file"), wxT(""), wxT(""), wxT(""),
-        _("Firebird database files (*.fdb, *.gdb)|*.fdb;*.gdb|All files (*.*)|*.*"), wxOPEN, this);
+        _("Firebird database files (*.fdb, *.gdb)|*.fdb;*.gdb|All files (*.*)|*.*"),
+#if wxCHECK_VERSION(2, 8, 0)
+        wxFD_OPEN, this);
+#else
+        wxOPEN, this);
+#endif
     if (!path.empty())
         text_ctrl_dbpath->SetValue(path);
 }
