@@ -1,27 +1,27 @@
 /*
-Copyright (c) 2004, 2005, 2006 The FlameRobin Development Team
+  Copyright (c) 2004-2008 The FlameRobin Development Team
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+  Permission is hereby granted, free of charge, to any person obtaining
+  a copy of this software and associated documentation files (the
+  "Software"), to deal in the Software without restriction, including
+  without limitation the rights to use, copy, modify, merge, publish,
+  distribute, sublicense, and/or sell copies of the Software, and to
+  permit persons to whom the Software is furnished to do so, subject to
+  the following conditions:
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+  The above copyright notice and this permission notice shall be included
+  in all copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: FieldPropertiesDialog.cpp 1235 2006-10-22 17:26:50Z mbabuskov $
+  $Id: FieldPropertiesDialog.cpp 1631 2008-02-02 22:25:30Z mapopa $
 
 */
 
@@ -40,7 +40,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <wx/gbsizer.h>
 
-#include "gui/ExecuteSqlFrame.h"
+#include "core/ArtProvider.h"
+#include "gui/ExecuteSql.h"
 #include "gui/FieldPropertiesDialog.h"
 #include "gui/StyleGuide.h"
 #include "metadata/column.h"
@@ -53,7 +54,7 @@ struct DatatypeProperties
 {
     //wxString name;    // doesn't work with Borland
     //wxChar name[];    // doesn't work with MSVC
-    wxChar* name;
+    const wxChar* name;
     bool hasSize;
     bool hasScale;
     bool isChar;
@@ -95,11 +96,7 @@ FieldPropertiesDialog::FieldPropertiesDialog(wxWindow* parent, Table* table,
     updateControls();
     layoutControls();
 
-    #include "column32.xpm"
-    wxBitmap bmp = wxBitmap(column_xpm);
-    wxIcon icon;
-    icon.CopyFromBitmap(bmp);
-    SetIcon(icon);
+    SetIcon(wxArtProvider::GetIcon(ART_Column, wxART_FRAME_ICON));
 }
 //-----------------------------------------------------------------------------
 void FieldPropertiesDialog::createControls()
@@ -271,7 +268,7 @@ void FieldPropertiesDialog::update()
 bool FieldPropertiesDialog::getDomainInfo(const wxString& domain,
     wxString& type, wxString& size, wxString& scale, wxString& charset)
 {
-    Database* db = tableM->getDatabase();
+    Database* db = tableM->findDatabase();
     if (db)
     {
         MetadataCollection<Domain>::const_iterator it;
@@ -450,10 +447,11 @@ void FieldPropertiesDialog::loadCharsets()
     choice_charset->Clear();
     choice_charset->Append(wxT("NONE"));
 
-    if (tableM && tableM->getDatabase())
+    Database* db = (tableM) ? tableM->findDatabase() : 0;
+    if (tableM && db)
     {
         vector<wxString> charsets;
-        tableM->getDatabase()->fillVector(charsets,
+        db->fillVector(charsets,
             wxT("select rdb$character_set_name from rdb$character_sets order by 1"));
         for (vector<wxString>::iterator it = charsets.begin();
             it != charsets.end(); ++it)
@@ -470,10 +468,11 @@ void FieldPropertiesDialog::loadCollations()
     choice_collate->Freeze();
     choice_collate->Clear();
 
-    if (tableM && tableM->getDatabase())
+    Database* db = (tableM) ? tableM->findDatabase() : 0;
+    if (tableM && db)
     {
         wxString charset(choice_charset->GetStringSelection());
-        vector<wxString> cols = tableM->getDatabase()->getCollations(charset);
+        vector<wxString> cols = db->getCollations(charset);
         for (vector<wxString>::iterator it = cols.begin(); it != cols.end(); it++)
             choice_collate->Append(*it);
     }
@@ -492,9 +491,9 @@ void FieldPropertiesDialog::loadDomains()
             choice_domain->SetSelection(0);
     }
 
-    if (tableM && tableM->getDatabase())
+    Database* db = (tableM) ? tableM->findDatabase() : 0;
+    if (tableM && db)
     {
-        Database* db = tableM->getDatabase();
         MetadataCollection<Domain>::const_iterator it;
         for (it = db->domainsBegin(); it != db->domainsEnd(); ++it)
         {
@@ -514,9 +513,9 @@ void FieldPropertiesDialog::loadGeneratorNames()
     choice_generator->Freeze();
     choice_generator->Clear();
 
-    if (tableM && tableM->getDatabase())
+    Database* db = (tableM) ? tableM->findDatabase() : 0;
+    if (tableM && db)
     {
-        Database* db = tableM->getDatabase();
         MetadataCollection<Generator>::const_iterator it;
         for (it = db->generatorsBegin(); it != db->generatorsEnd(); ++it)
             choice_generator->Append((*it).getName_());
@@ -724,9 +723,31 @@ END_EVENT_TABLE()
 void FieldPropertiesDialog::OnButtonEditDomainClick(wxCommandEvent&
     WXUNUSED(event))
 {
+    if (getIsNewDomainSelected())
+    {
+        // no domain selected
+        return;
+    }
+
+    // TODO:
     // create DomainPropertiesFrame & show it
     // when done, reload domain definition
     // updateDomainInfo(wx2std(cb_domains->GetValue()));
+
+    // Currently we just open the SQL editor and close this dialog
+    wxString domain = choice_domain->GetStringSelection();
+    Database* db = tableM->getDatabase(
+        wxT("FieldPropertiesDialog::OnButtonEditDomainClick"));
+    Domain *d = dynamic_cast<Domain *>(db->findByNameAndType(ntDomain,
+        domain));
+    if (!d)
+    {
+        // domain not found
+        return;
+    }
+
+    showSql(GetParent(), _("Alter domain"), db, d->getAlterSqlTemplate());
+    EndModal(wxID_CANCEL);
 }
 //-----------------------------------------------------------------------------
 void FieldPropertiesDialog::OnButtonOkClick(wxCommandEvent& WXUNUSED(event))
@@ -819,32 +840,25 @@ bool ColumnPropertiesHandler::handleURI(URI& uri)
         t = c->getTable();
     }
 
-	wxString statements, title;
-	{	// we want FPD to go out of scope and get destroyed since the action in
-		// ESF can destroy the field, and take down the FPD. Accessing FPD in turn
-		// leads to mysterious crash
-		FieldPropertiesDialog fpd(w, t, c);
-		// NOTE: this has been moved here from OnOkButtonClick() to make frame
-		//       activation work properly.  Basically activation of another
-		//       frame has to happen outside wxDialog::ShowModal(), because it
-		//       does at the end re-focus the last focused control, raising
-		//       the parent frame over the newly created sql execution frame
-		if (fpd.ShowModal() == wxID_OK)
-		{
-			statements = fpd.getStatementsToExecute();
-			title = fpd.getStatementTitle();
-		}
-	}
+    wxString statements, title;
+    {   // we want FPD to go out of scope and get destroyed since the action in
+        // ESF can destroy the field, and take down the FPD. Accessing FPD in turn
+        // leads to mysterious crash
+        FieldPropertiesDialog fpd(w, t, c);
+        // NOTE: this has been moved here from OnOkButtonClick() to make frame
+        //       activation work properly.  Basically activation of another
+        //       frame has to happen outside wxDialog::ShowModal(), because it
+        //       does at the end re-focus the last focused control, raising
+        //       the parent frame over the newly created sql execution frame
+        if (fpd.ShowModal() == wxID_OK)
+        {
+            statements = fpd.getStatementsToExecute();
+            title = fpd.getStatementTitle();
+        }
+    }
 
-	if (!statements.IsEmpty())
-	{
-		// create ExecuteSqlFrame with option to close at once
-		ExecuteSqlFrame *esf = new ExecuteSqlFrame(w, -1, title);
-		esf->setDatabase(t->getDatabase());
-		esf->setSql(statements);
-		esf->Show();
-		esf->executeAllStatements(true);
-	}
+    if (!statements.IsEmpty())
+        execSql(w, title, t->findDatabase(), statements, true);
     return true;
 }
 //-----------------------------------------------------------------------------
