@@ -1,27 +1,27 @@
 /*
-Copyright (c) 2004, 2005, 2006 The FlameRobin Development Team
+  Copyright (c) 2004-2008 The FlameRobin Development Team
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+  Permission is hereby granted, free of charge, to any person obtaining
+  a copy of this software and associated documentation files (the
+  "Software"), to deal in the Software without restriction, including
+  without limitation the rights to use, copy, modify, merge, publish,
+  distribute, sublicense, and/or sell copies of the Software, and to
+  permit persons to whom the Software is furnished to do so, subject to
+  the following conditions:
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+  The above copyright notice and this permission notice shall be included
+  in all copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: MetadataItemPropertiesFrame.cpp 1227 2006-10-02 17:08:45Z mbabuskov $
+  $Id: MetadataItemPropertiesFrame.cpp 1647 2008-03-11 13:03:05Z mghie $
 
 */
 
@@ -42,6 +42,10 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <wx/file.h>
 #include <wx/filedlg.h>
 
+#if wxCHECK_VERSION(2, 8, 0)
+#include <wx/tipwin.h>
+#endif
+
 #include <fstream>
 #include <iomanip>
 #include <sstream>
@@ -49,7 +53,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "config/Config.h"
 #include "core/FRError.h"
 #include "core/StringUtils.h"
-#include "dberror.h"
+#include "engine/MetadataLoader.h"
 #include "framemanager.h"
 #include "frutils.h"
 #include "gui/MetadataItemPropertiesFrame.h"
@@ -58,37 +62,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "metadata/CreateDDLVisitor.h"
 #include "metadata/database.h"
 #include "metadata/exception.h"
+#include "metadata/server.h"
 #include "metadata/table.h"
 #include "metadata/view.h"
 #include "urihandler.h"
 //-----------------------------------------------------------------------------
-wxString loadHtmlFile(const wxString& filename)
-{
-    wxFileName localFileName = filename;
-    if (!localFileName.FileExists())
-    {
-        wxString msg;
-        msg.Printf(_("The file \"%s\" does not exist."),
-            localFileName.GetFullPath().c_str());
-        throw FRError(msg);
-    }
-
-    std::ifstream file(wx2std(filename).c_str()); // read entire file into wxString buffer
-    if (!file)
-    {
-        wxString msg;
-        msg.Printf(_("The file \"%s\" cannot be opened."),
-            filename.c_str());
-        throw FRError(msg);
-    }
-
-    std::stringstream ss;
-    ss << file.rdbuf();
-    wxString s(std2wx(ss.str()));
-    file.close();
-    return s;
-}
-//-----------------------------------------------------------------------------
 //! MetadataItemPropertiesFrame class
 MetadataItemPropertiesFrame::MetadataItemPropertiesFrame(wxWindow* parent,
         MetadataItem *object)
@@ -105,18 +83,30 @@ MetadataItemPropertiesFrame::MetadataItemPropertiesFrame(wxWindow* parent,
         return;
     }
 
-    html_window = new PrintableHtmlWindow(this);
-    CreateStatusBar();
+    html_window = new PrintableHtmlWindow(this, HtmlWindowID);
+
+    wxStatusBar *sb = CreateStatusBar();
+    Database* d = objectM->findDatabase();
+    if (d)  // server property page doesn't have a database, so don't crash
+    {
+        wxString s = d->getUsername() + wxT("@") + d->getConnectionString()
+            + wxT(" (") + d->getConnectionCharset() + wxT(")");
+        sb->SetStatusText(s);
+    }
+    else
+        sb->SetStatusText(objectM->getPrintableName());
+
+    // start a transaction for metadata loading and lock the object
+    MetadataLoaderTransaction tr((d) ? d->getMetadataLoader() : 0);
+    SubjectLocker lock(objectM);
 
     // request initial rendering
     requestLoadPage(true);
     objectM->attachObserver(this);
-    update();
 
     wxString objName(objectM->getName_());
     SetTitle(wxString::Format(_("%s: Properties"), objName.c_str()));
     html_window->SetRelatedFrame(this, objName + wxT(": %s"));
-    html_window->SetRelatedStatusBar(0);
 
     wxBitmap bmp = getImage32(objectM->getType());
     wxIcon icon;
@@ -214,6 +204,12 @@ void MetadataItemPropertiesFrame::loadPage()
     }
 
     wxBusyCursor bc;
+
+    // start a transaction for metadata loading and lock the object
+    Database* d = objectM->findDatabase();
+    MetadataLoaderTransaction tr((d) ? d->getMetadataLoader() : 0);
+    SubjectLocker lock(objectM);
+
     processHtmlFile(htmlpage);  // load HTML template, parse, and fill the HTML control
 }
 //-----------------------------------------------------------------------------
@@ -284,7 +280,7 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
             case ntRole:
                 pages.push_back(wxT("DDL"));
         };
-        wxString page = loadHtmlFile(config().getHtmlTemplatesPath()
+        wxString page = loadEntireFile(config().getHtmlTemplatesPath()
             + wxT("header.html"));
         bool first = true;
         while (!page.Strip().IsEmpty())
@@ -351,6 +347,19 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
             htmlpage << u->groupidM;
     }
 
+    else if (cmd == wxT("owner_name"))
+    {
+        wxString name;
+        if (Relation* r = dynamic_cast<Relation*>(object))
+            name = r->getOwner();
+        else if (Procedure* p = dynamic_cast<Procedure*>(object))
+            name = p->getOwner();
+        else if (Role* r = dynamic_cast<Role*>(object))
+            name = r->getOwner();
+        if (!name.empty())
+            htmlpage += escapeHtmlChars(name, false);
+    }
+
     else if (cmd == wxT("object_description"))
     {
         if (object->isDescriptionAvailable())
@@ -387,7 +396,8 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
         if (!r)
             return;
         std::vector<MetadataItem*> tmp;
-        if (r->checkAndLoadColumns() && r->getChildren(tmp))
+        r->checkAndLoadColumns();
+        if (r->getChildren(tmp))
         {
             for (std::vector<MetadataItem*>::iterator it = tmp.begin(); it != tmp.end(); ++it)
                 processHtmlCode(htmlpage, suffix, *it);
@@ -400,19 +410,13 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
         if (!r)
             return;
         std::vector<Trigger*> tmp;
-        bool result;
         if (suffix.substr(0, 5) == wxT("after"))
-            result = r->getTriggers(tmp, Trigger::afterTrigger);
+            r->getTriggers(tmp, Trigger::afterTrigger);
         else
-            result = r->getTriggers(tmp, Trigger::beforeTrigger);
+            r->getTriggers(tmp, Trigger::beforeTrigger);
         suffix.erase(0, 5);
-        if (result)
-        {
-            for (std::vector<Trigger*>::iterator it = tmp.begin(); it != tmp.end(); ++it)
-                processHtmlCode(htmlpage, suffix, *it);
-        }
-        else
-            ::wxMessageBox(lastError().getMessage(), _("Error"), wxOK);
+        for (std::vector<Trigger*>::iterator it = tmp.begin(); it != tmp.end(); ++it)
+            processHtmlCode(htmlpage, suffix, *it);
     }
 
     else if (cmd == wxT("depends_on") || cmd == wxT("depend_of"))
@@ -421,13 +425,12 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
         if (!m)
             return;
         std::vector<Dependency> tmp;
-        if (m->getDependencies(tmp, cmd == wxT("depends_on")))
+        m->getDependencies(tmp, cmd == wxT("depends_on"));
+        for (std::vector<Dependency>::iterator it = tmp.begin();
+            it != tmp.end(); ++it)
         {
-            for (std::vector<Dependency>::iterator it = tmp.begin(); it != tmp.end(); ++it)
-                processHtmlCode(htmlpage, suffix, &(*it));
+            processHtmlCode(htmlpage, suffix, &(*it));
         }
-        else
-            ::wxMessageBox(lastError().getMessage(), _("Error"), wxOK);
     }
 
     else if (cmd == wxT("dependency_columns"))
@@ -539,8 +542,8 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
                 }
 
                 // wxHTML doesn't support TITLE or ALT property of IMG tag so
-                // we show grantor like this (display in status bar)
-                htmlpage += wxT("<a href=\"info://      Granted by ") + (*it).grantor
+                // we use our custom 'link hower' handler to show it
+                htmlpage += wxT("<a href=\"info://Granted by ") + (*it).grantor
                     + wxT("\">");
 
                 htmlpage += wxT("<img src=\"") +
@@ -549,12 +552,7 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
                     htmlpage += wxT("ok2.png\"");
                 else
                     htmlpage += wxT("ok.png\"");
-                // wxHTML doesn't seem to support this
-                htmlpage += wxT(" TITLE=\"Granted by ") + (*it).grantor
-                    + wxT("\">");
-
-                // see wxHTML comment above
-                htmlpage += wxT("</a>");
+                htmlpage += wxT("\"></a>");
 
                 if ((*it).columns.size())
                 {
@@ -620,8 +618,8 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
     {
         Column* c = dynamic_cast<Column*>(object);
         if (c)
-        {
-            htmlpage += c->getDatatype();
+        {   // needs newlines escaped for computed column source
+            htmlpage += escapeHtmlChars(c->getDatatype());
             // TODO: make the domain name (if any) a link to the domain's property page?
         }
     }
@@ -657,13 +655,14 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
 
         SubjectLocker locker(p);
         std::vector<MetadataItem*> tmp;
-        if (p->checkAndLoadParameters() && p->getChildren(tmp))
+        p->checkAndLoadParameters();
+        if (p->getChildren(tmp))
         {
-            ParameterType pt = (cmd == wxT("input_parameters")) ? ptInput : ptOutput;
+            bool parOut = (cmd == wxT("output_parameters"));
             std::vector<MetadataItem*>::iterator it;
             for (it = tmp.begin(); it != tmp.end(); ++it)
             {
-                if ((dynamic_cast<Parameter*>(*it))->getParameterType() == pt)
+                if ((dynamic_cast<Parameter*>(*it))->isOutputParameter() == parOut)
                     processHtmlCode(htmlpage, suffix, *it);
             }
         }
@@ -672,36 +671,25 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
     else if (cmd == wxT("view_source"))
     {
         View* v = dynamic_cast<View*>(object);
-        wxString src;
-        if (!v || !v->getSource(src))
+        if (!v)
             return;
-        htmlpage += escapeHtmlChars(src, false);
+        htmlpage += escapeHtmlChars(v->getSource(), false);
     }
 
     else if (cmd == wxT("procedure_source"))
     {
         Procedure* p = dynamic_cast<Procedure*>(object);
-        wxString src;
-        if (!p || !p->getSource(src))
+        if (!p)
             return;
-        htmlpage += escapeHtmlChars(src, false);
-    }
-
-    else if (cmd == wxT("role_owner"))
-    {
-        Role* r = dynamic_cast<Role*>(object);
-        if (!r)
-            return;
-        htmlpage += escapeHtmlChars(r->getRoleOwner(), false);
+        htmlpage += escapeHtmlChars(p->getSource(), false);
     }
 
     else if (cmd == wxT("trigger_source"))
     {
         Trigger* t = dynamic_cast<Trigger*>(object);
-        wxString src;
-        if (!t || !t->getSource(src))
+        if (!t)
             return;
-        htmlpage += escapeHtmlChars(src, false);
+        htmlpage += escapeHtmlChars(t->getSource(), false);
     }
 
     else if (cmd == wxT("trigger_info"))
@@ -710,8 +698,9 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
         wxString object, type;
         bool active;
         int position;
-        if (!t || !t->getTriggerInfo(object, active, position, type))
+        if (!t)
             return;
+        t->getTriggerInfo(object, active, position, type);
         wxString s(active ? wxT("Active ") : wxT("Inactive "));
         s << type << wxT(" trigger for ") << object << wxT(" at position ") << position;
         htmlpage += escapeHtmlChars(s, false);
@@ -839,13 +828,16 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
         if (!d)
             return;
 
-        int size = d->getInfo()->getSize();
-        if (size > 1048576)
-            htmlpage += wxString::Format(wxT("%0.2fGB"), size/1048576.0);
-        else if (size > 1024)
-            htmlpage += wxString::Format(wxT("%0.2fMB"), size/1024.0);
+        float size = (float)d->getInfo()->getPageSize() * d->getInfo()->getPages();
+        const float kilo = 1024.0;
+        const float mega = kilo * kilo;
+        const float giga = kilo * mega;
+        if (size >= giga)
+            htmlpage += wxString::Format(wxT("%0.2fGB"), size / giga);
+        else if (size >= mega)
+            htmlpage += wxString::Format(wxT("%0.2fMB"), size / mega);
         else
-            htmlpage << size << wxT("kB");
+            htmlpage += wxString::Format(wxT("%0.2fkB"), size / kilo);
     }
     else if (cmd == wxT("forced_writes"))
     {
@@ -881,6 +873,11 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
             return;
 
         htmlpage += wxString() << d->getInfo()->getODS();
+        if (d->getInfo()->getODSMinor())
+        {
+            htmlpage += wxT(".");
+            htmlpage += wxString() << d->getInfo()->getODSMinor();
+        }
     }
     else if (cmd == wxT("oldest_transaction"))
     {
@@ -987,7 +984,7 @@ void MetadataItemPropertiesFrame::processHtmlCode(wxString& htmlpage, wxString h
 void MetadataItemPropertiesFrame::processHtmlFile(wxString fileName)
 {
     wxString htmlpage;
-    processHtmlCode(htmlpage, loadHtmlFile(fileName));
+    processHtmlCode(htmlpage, loadEntireFile(fileName));
 
     int x = 0, y = 0;
     html_window->GetViewStart(&x, &y);         // save scroll position
@@ -1076,17 +1073,51 @@ void MetadataItemPropertiesFrame::update()
     requestLoadPage(false);
 }
 //-----------------------------------------------------------------------------
+#if wxCHECK_VERSION(2, 8, 0)
+BEGIN_EVENT_TABLE(MetadataItemPropertiesFrame, BaseFrame)
+    EVT_HTML_CELL_HOVER(MetadataItemPropertiesFrame::HtmlWindowID,
+        MetadataItemPropertiesFrame::OnHtmlCellHover)
+END_EVENT_TABLE()
+#endif
+//-----------------------------------------------------------------------------
 void MetadataItemPropertiesFrame::OnIdle(wxIdleEvent& WXUNUSED(event))
 {
-    FR_TRY
-
     Disconnect(wxID_ANY, wxEVT_IDLE);
+    wxBusyCursor bc;
     htmlReloadRequestedM = false;
     loadPage();
-
-    FR_CATCH
 }
 //-----------------------------------------------------------------------------
+#if wxCHECK_VERSION(2, 8, 0)
+void MetadataItemPropertiesFrame::OnHtmlCellHover(wxHtmlCellEvent& event)
+{
+    wxHtmlCell *c = event.GetCell();
+    if (!c)
+        return;
+    wxHtmlLinkInfo *lnk = c->GetLink();
+    if (!lnk)
+        return;
+
+    wxString addr = lnk->GetHref();
+    URI uri(addr);
+    if (uri.protocol == wxT("info"))    // special
+    {
+        //      GetStatusBar()->SetStatusText(uri.action);
+
+        // I'm having a hard time trying to convert this to screen coordinates
+        // since parent's coords cannot be retrieved(?)
+        //wxRect r(c->GetPosX(), c->GetPosY(), c->GetWidth(), c->GetHeight());
+
+        // M.B. So I decided to use a 21x9 box around the mouse
+        wxRect r(::wxGetMousePosition().x - 10, ::wxGetMousePosition().y - 4,
+            21, 9);
+
+        wxTipWindow *tw = new wxTipWindow(this, uri.action);
+        tw->SetBoundingRect(r);
+    }
+}
+#endif
+//-----------------------------------------------------------------------------
 //! PageHandler class
 class PageHandler: public URIHandler
 {
@@ -1144,7 +1175,7 @@ bool PropertiesHandler::handleURI(URI& uri)
     MetadataItemPropertiesFrame* parent = dynamic_cast<MetadataItemPropertiesFrame*>(getWindow(uri));
     if (!parent)
         return true;
-    Database* d = parent->getObservedObject()->getDatabase();
+    Database* d = parent->getObservedObject()->findDatabase();
     if (!d)
         return true;
     NodeType n = getTypeByName(uri.getParam(wxT("object_type")));
