@@ -1,27 +1,27 @@
 /*
-Copyright (c) 2004, 2005, 2006 The FlameRobin Development Team
+  Copyright (c) 2004-2008 The FlameRobin Development Team
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+  Permission is hereby granted, free of charge, to any person obtaining
+  a copy of this software and associated documentation files (the
+  "Software"), to deal in the Software without restriction, including
+  without limitation the rights to use, copy, modify, merge, publish,
+  distribute, sublicense, and/or sell copies of the Software, and to
+  permit persons to whom the Software is furnished to do so, subject to
+  the following conditions:
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+  The above copyright notice and this permission notice shall be included
+  in all copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: PreferencesDialog.cpp 1248 2006-11-14 16:11:21Z mghie $
+  $Id: PreferencesDialog.cpp 1631 2008-02-02 22:25:30Z mapopa $
 
 */
 
@@ -41,16 +41,17 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <algorithm>
 #include <vector>
 
+#include <wx/artprov.h>
 #include <wx/bookctrl.h>
 #include <wx/tokenzr.h>
 #include <wx/wfstream.h>
 #include <wx/xml/xml.h>
 
 #include "config/Config.h"
+#include "core/ArtProvider.h"
 #include "frutils.h"
 #include "gui/PreferencesDialog.h"
 #include "gui/StyleGuide.h"
-#include "images.h"
 //-----------------------------------------------------------------------------
 static bool hasParamNode(wxXmlNode* node, const wxString& param)
 {
@@ -89,7 +90,7 @@ wxString getPlatformName()
 #endif
 }
 //-----------------------------------------------------------------------------
-static void processPlatformProperty(wxXmlNode *node)
+static void processPlatformAttribute(wxXmlNode *node)
 {
     wxString s;
     bool isok;
@@ -98,7 +99,11 @@ static void processPlatformProperty(wxXmlNode *node)
     while (c)
     {
         isok = false;
+#if wxCHECK_VERSION(2, 9, 0)
+        if (!c->GetAttribute(wxT("platform"), &s))
+#else
         if (!c->GetPropVal(wxT("platform"), &s))
+#endif
             isok = true;
         else
         {
@@ -113,7 +118,7 @@ static void processPlatformProperty(wxXmlNode *node)
 
         if (isok)
         {
-            processPlatformProperty(c);
+            processPlatformAttribute(c);
             c = c->GetNext();
         }
         else
@@ -327,7 +332,7 @@ PreferencesDialog::PreferencesDialog(wxWindow* parent, const wxString& title,
         wxSUNKEN_BORDER | wxTR_DEFAULT_STYLE | wxTR_HAS_BUTTONS | wxTR_HIDE_ROOT);
     panel_categ = new wxPanel(getControlsPanel(), wxID_ANY, wxDefaultPosition,
         wxDefaultSize, wxSUNKEN_BORDER);
-    static_text_categ = new wxStaticText(panel_categ, wxID_ANY, wxEmptyString);
+    static_text_categ = new wxStaticText(panel_categ, wxID_ANY, wxT("Dummy min size text"));
     bookctrl_1 = new Optionbook(getControlsPanel(), ID_bookctrl_panes,
         wxDefaultPosition, wxDefaultSize);
 
@@ -432,9 +437,15 @@ void PreferencesDialog::layout()
     sizerRight->Add(bookctrl_1, 1, wxEXPAND);
 
     wxBoxSizer* sizerControls = new wxBoxSizer(wxHORIZONTAL);
+#if wxCHECK_VERSION(2, 8, 0) && !defined(__WXMAC__)
+    sizerControls->Add(treectrl_1, 0, wxEXPAND);
+    sizerControls->Add(styleguide().getUnrelatedControlMargin(wxHORIZONTAL), 0);
+    sizerControls->Add(sizerRight, 1, wxEXPAND);
+#else
     sizerControls->Add(treectrl_1, 2, wxEXPAND);
     sizerControls->Add(styleguide().getUnrelatedControlMargin(wxHORIZONTAL), 0);
     sizerControls->Add(sizerRight, 5, wxEXPAND);
+#endif
 
     // create sizer for buttons -> styleguide class will align it correctly
     wxSizer* sizerButtons = styleguide().createButtonSizer(button_save, button_cancel);
@@ -468,7 +479,7 @@ void PreferencesDialog::loadDescriptionFile(const wxFileName& filename)
             filename.GetFullPath().c_str());
         return;
     }
-    processPlatformProperty(xmlr);
+    processPlatformAttribute(xmlr);
     debugDescriptionM = hasParamNode(xmlr, wxT("debug"));
 
     wxTreeItemId root = treectrl_1->AddRoot(wxEmptyString);
@@ -541,6 +552,11 @@ bool PreferencesDialog::parseDescriptionNode(wxTreeItemId parent, wxXmlNode* xml
         if (description.IsEmpty())
             description = caption;
     }
+#if wxCHECK_VERSION(2, 8, 0)
+    treectrl_1->ExpandAllChildren(item);
+#else
+    treectrl_1->Expand(item);
+#endif
 
     // add all settings controls of this page to a sizer
     wxBoxSizer* sizerPage = new wxBoxSizer(wxVERTICAL);
@@ -557,7 +573,11 @@ bool PreferencesDialog::parseDescriptionNode(wxTreeItemId parent, wxXmlNode* xml
 bool PreferencesDialog::parseDescriptionSetting(wxPanel* page, wxXmlNode* xmln,
     PrefDlgSetting* enabledby)
 {
+#if wxCHECK_VERSION(2, 9, 0)
+    wxString type(xmln->GetAttribute(wxT("type"), wxEmptyString));
+#else
     wxString type(xmln->GetPropVal(wxT("type"), wxEmptyString));
+#endif
     PrefDlgSetting* setting = createPrefDlgSetting(page, type, enabledby);
     // ignore unknown settings unless debug mode is active
     if (setting == 0)
@@ -622,15 +642,20 @@ void PreferencesDialog::selectPage(int index)
 void PreferencesDialog::setProperties()
 {
     // setup image list for tree control
-    imageListM.Create(16, 16);
-    imageListM.Add(getImage(ntColumn));
-    imageListM.Add(getImage(ntProcedure));
-    imageListM.Add(getImage(ntTrigger));
-    imageListM.Add(getImage(ntUnknown));
-    imageListM.Add(getImage(ntTable));
-    imageListM.Add(getImage(ntDomain));
+    wxSize sz(16, 16);
+    imageListM.Create(sz.GetWidth(), sz.GetHeight());
+    imageListM.Add(wxArtProvider::GetIcon(ART_Column, wxART_OTHER, sz));
+    imageListM.Add(wxArtProvider::GetIcon(ART_Procedure, wxART_OTHER, sz));
+    imageListM.Add(wxArtProvider::GetIcon(ART_Trigger, wxART_OTHER, sz));
+    imageListM.Add(wxArtProvider::GetIcon(ART_Object, wxART_OTHER, sz));
+    imageListM.Add(wxArtProvider::GetIcon(ART_Table, wxART_OTHER, sz));
+    imageListM.Add(wxArtProvider::GetIcon(ART_Domain, wxART_OTHER, sz));
     treectrl_1->SetImageList(&imageListM);
 
+#if wxCHECK_VERSION(2, 8, 0)
+    treectrl_1->SetQuickBestSize(false);
+#endif
+
     // show category description in colors for highlighted text
     panel_categ->SetBackgroundColour(
         wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT));
