@@ -21,7 +21,7 @@ TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: PrivilegesDialog.cpp 1235 2006-10-22 17:26:50Z mbabuskov $
+  $Id: PrivilegesDialog.cpp 1598 2007-12-20 18:47:29Z babuskov $
 
 */
 //-----------------------------------------------------------------------------
@@ -37,7 +37,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <wx/gbsizer.h>
 
-#include "gui/ExecuteSqlFrame.h"
+#include "gui/ExecuteSql.h"
 #include "gui/PrivilegesDialog.h"
 #include "gui/StyleGuide.h"
 #include "metadata/collection.h"
@@ -56,7 +56,7 @@ PrivilegesDialog::PrivilegesDialog(wxWindow *parent, MetadataItem *object,
     // since not all objects are created by that time - event handling code
     // crashes
     inConstructor = true;
-    databaseM = object->getDatabase();
+    databaseM = object->findDatabase();
 
     wxBoxSizer *innerSizer = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer *topSizer = new wxBoxSizer(wxHORIZONTAL);
@@ -295,8 +295,8 @@ PrivilegesDialog::PrivilegesDialog(wxWindow *parent, MetadataItem *object,
     }
     choice_memberof->Enable(false);
     fgSizer4->Add(choice_memberof, 0, wxEXPAND, 0);
-	privilegesSizer->Add(fgSizer4, 1, wxEXPAND|wxALL, 5);
-	privilegesPanel->SetSizer(privilegesSizer);
+    privilegesSizer->Add(fgSizer4, 1, wxEXPAND|wxALL, 5);
+    privilegesPanel->SetSizer(privilegesSizer);
     topSizer->Add(privilegesPanel, 1, wxEXPAND, 0);
     // PRIVILEGES done
 
@@ -433,8 +433,11 @@ void PrivilegesDialog::updateControls()
     }
 
     wxString grantee;
+    // some usernames need quoting
     if (radiobtn_user->GetValue())
-        grantee = textctrl_user->GetValue();
+    {
+        grantee = Identifier::userString(textctrl_user->GetValue());
+    }
     else if (radiobtn_trigger->GetValue())
     {
         grantee = wxT("TRIGGER ") + Identifier(
@@ -504,7 +507,7 @@ void PrivilegesDialog::updateControls()
 wxString PrivilegesDialog::getSqlStatements()
 {
     wxString stmt;
-    for (int i = 0; i < listbox_statements->GetCount(); i++)
+    for (int i = 0; i < (int)listbox_statements->GetCount(); i++)
         stmt << listbox_statements->GetString(i) << wxT(";\n");
     return stmt;
 }
@@ -610,13 +613,8 @@ bool ManagePrivilegesHandler::handleURI(URI& uri)
         // nothing to be done
         if (!statements.IsEmpty())
         {
-            // create ExecuteSqlFrame with option to close at once
-            ExecuteSqlFrame *esf = new ExecuteSqlFrame(w, -1,
-                _("Grant And Revoke Privileges"));
-            esf->setDatabase(m->getDatabase());
-            esf->setSql(statements);
-            esf->executeAllStatements(true);
-            esf->Show();
+            execSql(w, _("Grant And Revoke Privileges"), m->findDatabase(),
+                statements, true);
         }
     }
     return true;
