@@ -1,27 +1,27 @@
 /*
-Copyright (c) 2004, 2005, 2006 The FlameRobin Development Team
+  Copyright (c) 2004-2008 The FlameRobin Development Team
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+  Permission is hereby granted, free of charge, to any person obtaining
+  a copy of this software and associated documentation files (the
+  "Software"), to deal in the Software without restriction, including
+  without limitation the rights to use, copy, modify, merge, publish,
+  distribute, sublicense, and/or sell copies of the Software, and to
+  permit persons to whom the Software is furnished to do so, subject to
+  the following conditions:
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+  The above copyright notice and this permission notice shall be included
+  in all copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: RestoreFrame.cpp 1163 2006-06-08 11:43:04Z mghie $
+  $Id: RestoreFrame.cpp 1631 2008-02-02 22:25:30Z mapopa $
 
 */
 
@@ -48,9 +48,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "config/Config.h"
 #include "core/StringUtils.h"
 #include "frutils.h"
+#include "gui/controls/DndTextControls.h"
 #include "gui/controls/LogTextControl.h"
 #include "gui/RestoreFrame.h"
 #include "gui/StyleGuide.h"
+#include "gui/UsernamePasswordDialog.h"
 #include "metadata/database.h"
 #include "metadata/server.h"
 //-----------------------------------------------------------------------------
@@ -203,8 +205,8 @@ void RestoreFrame::createControls()
     panel_controls = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxDefaultSize,
         wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_FULL_REPAINT_ON_RESIZE);
     label_filename = new wxStaticText(panel_controls, wxID_ANY, _("Backup file:"));
-    text_ctrl_filename = new wxTextCtrl(panel_controls, ID_text_ctrl_filename,
-        wxEmptyString);
+    text_ctrl_filename = new FileTextControl(panel_controls,
+        ID_text_ctrl_filename, wxEmptyString);
     button_browse = new wxButton(panel_controls, ID_button_browse, _("..."),
         wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
 
@@ -401,8 +403,13 @@ void RestoreFrame::OnBrowseButtonClick(wxCommandEvent& WXUNUSED(event))
 {
     wxFileName origName(text_ctrl_filename->GetValue());
     wxString filename = ::wxFileSelector(_("Select Backup File"),
-        origName.GetPath(), origName.GetName(), wxEmptyString,
-        _("All files (*.*)|*.*"), wxOPEN, this);
+        origName.GetPath(), origName.GetFullName(), wxT("*.fbk"),
+        _("Backup file (*.fbk, *.gbk)|*.fbk;*.gbk|All files (*.*)|*.*"),
+#if wxCHECK_VERSION(2, 8, 0)
+        wxFD_OPEN, this);
+#else
+        wxOPEN, this);
+#endif
     if (!filename.empty())
         text_ctrl_filename->SetValue(filename);
 }
@@ -412,21 +419,21 @@ void RestoreFrame::OnStartButtonClick(wxCommandEvent& WXUNUSED(event))
     verboseMsgsM = checkbox_showlog->IsChecked();
     clearLog();
 
-    // TODO: create a global helper function
-    //   bool getDatabasePassword(wxFrame* parent, Database* db, wxString password);
-    // this would simplify the next lines to
-    //   if (!getDatabasePassword(this, databaseM, password))
-    //       return;
-
+    wxString username = databaseM->getUsername();
     wxString password = databaseM->getDecryptedPassword();
     if (password.empty())
     {
-        wxString msg;
-        msg.Printf(_("Enter password for user %s:"), databaseM->getUsername().c_str());
-        password = ::wxGetPasswordFromUser(msg, _("Connecting To Server"));
-        if (password.empty())
-            return;
+        UsernamePasswordDialog upd(this, _("Database Credentials"),
+            username, false, // allow different username
+            _("Please enter a valid username and password:"));
+        if (upd.ShowModal() == wxID_OK)
+        {
+            username = upd.getUsername();
+            password = upd.getPassword();
+        }
     }
+    if (password.empty())
+        return;
 
     int flags = (int)IBPP::brVerbose; // this will be ORed in anyway
     if (checkbox_replace->IsChecked())
@@ -447,7 +454,7 @@ void RestoreFrame::OnStartButtonClick(wxCommandEvent& WXUNUSED(event))
         pagesize = 0;
 
     RestoreThread* thread = new RestoreThread(this, serverM->getConnectionString(),
-        databaseM->getUsername(), password, text_ctrl_filename->GetValue(),
+        username, password, text_ctrl_filename->GetValue(),
         databaseM->getPath(), pagesize, (IBPP::BRF)flags);
     startThread(thread);
     updateControls();
