@@ -1,27 +1,27 @@
 /*
-Copyright (c) 2004, 2005, 2006 The FlameRobin Development Team
+  Copyright (c) 2004-2008 The FlameRobin Development Team
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+  Permission is hereby granted, free of charge, to any person obtaining
+  a copy of this software and associated documentation files (the
+  "Software"), to deal in the Software without restriction, including
+  without limitation the rights to use, copy, modify, merge, publish,
+  distribute, sublicense, and/or sell copies of the Software, and to
+  permit persons to whom the Software is furnished to do so, subject to
+  the following conditions:
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+  The above copyright notice and this permission notice shall be included
+  in all copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: DataGrid.cpp 1233 2006-10-21 13:26:25Z mbabuskov $
+  $Id: DataGrid.cpp 1631 2008-02-02 22:25:30Z mapopa $
 
 */
 
@@ -49,17 +49,16 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "core/FRError.h"
 #include "core/StringUtils.h"
 #include "gui/AdvancedMessageDialog.h"
-#include "gui/ExecuteSqlFrame.h"
+#include "gui/CommandIds.h"
 #include "gui/controls/DataGrid.h"
 #include "gui/controls/DataGridTable.h"
 #include "metadata/database.h"
 #include "metadata/table.h"
-
 //-----------------------------------------------------------------------------
 DataGrid::DataGrid(wxWindow* parent, wxWindowID id)
     : wxGrid(parent, id)
 {
-    EnableEditing(false);
+    EnableEditing(true);
     SetColLabelValue(0, wxT(""));
     SetRowLabelSize(50);
     DisableDragRowSize();
@@ -100,6 +99,43 @@ void DataGrid::copyToClipboard(const wxString cbText)
     wxTheClipboard->Close();
 }
 //-----------------------------------------------------------------------------
+void DataGrid::extendSelection(int direction)
+{
+    // extend selection of cell selection blocks
+    wxGridCellCoordsArray tlCells(GetSelectionBlockTopLeft());
+    wxGridCellCoordsArray brCells(GetSelectionBlockBottomRight());
+    wxASSERT(tlCells.GetCount() == brCells.GetCount());
+    for (size_t i = 0; i < tlCells.GetCount(); ++i)
+    {
+        wxGridCellCoords tl = tlCells[i];
+        wxGridCellCoords br = brCells[i];
+        if (direction & wxHORIZONTAL)
+        {
+            for (int r = tl.GetRow(); r <= br.GetRow(); ++r)
+                SelectRow(r, true);
+        }
+        if (direction & wxVERTICAL)
+        {
+            for (int c = tl.GetCol(); c <= br.GetCol(); ++c)
+                SelectCol(c, true);
+        }
+    }
+
+    // extend selection of single selected cells
+    // if no other selection then extend selection of active cell
+    wxGridCellCoordsArray selCells(GetSelectedCells());
+    if (!tlCells.GetCount() && !brCells.GetCount() && !selCells.GetCount())
+        selCells.Add(wxGridCellCoords(GetGridCursorRow(), GetGridCursorCol()));
+    for (size_t i = 0; i < selCells.GetCount(); ++i)
+    {
+        wxGridCellCoords cc = selCells[i];
+        if (direction & wxHORIZONTAL)
+            SelectRow(cc.GetRow(), true);
+        if (direction & wxVERTICAL)
+            SelectCol(cc.GetCol(), true);
+    }
+}
+//-----------------------------------------------------------------------------
 void DataGrid::fetchData(wxMBConv* conv)
 {
     DataGridTable* table = getDataGridTable();
@@ -110,13 +146,19 @@ void DataGrid::fetchData(wxMBConv* conv)
     BeginBatch();
     table->initialFetch(conv);
 
-    for (int i = 1; i <= table->GetNumberCols(); i++)
+    for (int i = 0; i < table->GetNumberCols(); i++)
     {
         wxGridCellAttr *ca = new wxGridCellAttr;
         ca->SetAlignment(
             (table->isNumericColumn(i)) ? wxALIGN_RIGHT : wxALIGN_LEFT,
-            wxALIGN_CENTRE);
-        SetColAttr(i-1, ca);
+            wxALIGN_TOP);
+        if (table->isReadonlyColumn(i))
+        {
+            ca->SetReadOnly(true);
+            ca->SetBackgroundColour(table->getReadonlyColour());
+        }
+        ca->SetOverflow(false);
+        SetColAttr(i, ca);
     }
     AutoSizeColumns();
     EndBatch();
@@ -154,18 +196,21 @@ void DataGrid::notifyIfUnfetchedData()
 void DataGrid::showPopMenu(wxPoint cursorPos)
 {
     wxMenu m(0);
-    m.Append(ID_MENU_FETCHALL, _("Fetch all records"));
-    m.Append(ID_MENU_CANCELFETCHALL, _("Stop fetching all records"));
+
+    // TODO: merge this with ExecuteSqlFrame's menu
+    m.Append(Cmds::DataGrid_FetchAll, _("Fetch all records"));
+    m.Append(Cmds::DataGrid_CancelFetchAll, _("Stop fetching all records"));
     m.AppendSeparator();
 
-    m.Append(ID_MENU_COPYTOCLIPBOARD, _("Copy"));
-    m.Append(ID_MENU_COPYTOCLIPBOARDASINSERT, _("Copy as INSERT statements"));
-    m.Append(ID_MENU_COPYTOCLIPBOARDASUPDATE, _("Copy as UPDATE statements"));
-    m.Append(ID_MENU_SAVEASHTML, _("Save as HTML file..."));
+    m.Append(Cmds::DataGrid_Copy, _("Copy"));
+    m.Append(Cmds::DataGrid_Copy_as_insert, _("Copy as INSERT statements"));
+    m.Append(Cmds::DataGrid_Copy_as_update, _("Copy as UPDATE statements"));
+    m.Append(Cmds::DataGrid_Save_as_html, _("Save as HTML file..."));
+    m.Append(Cmds::DataGrid_Save_as_csv, _("Save as CSV file..."));
     m.AppendSeparator();
 
-    m.Append(ID_MENU_LABELFONT, _("Set header font"));
-    m.Append(ID_MENU_CELLFONT, _("Set cell font"));
+    m.Append(Cmds::DataGrid_Set_header_font, _("Set header font"));
+    m.Append(Cmds::DataGrid_Set_cell_font, _("Set cell font"));
     PopupMenu(&m, cursorPos);
 }
 //-----------------------------------------------------------------------------
@@ -179,84 +224,20 @@ void DataGrid::updateRowHeights()
 #endif
 
     wxScreenDC dc;
-    // adjust height of grid header row (extra 2 pixels for border)
+    // adjust height of grid header row (extra 3 pixels for border)
     dc.SetFont(GetLabelFont());
-    SetColLabelSize(dc.GetCharHeight() + extraHeight + 2);
+    SetColLabelSize(dc.GetCharHeight() + extraHeight + 3);
 
-    // adjust height of rows, and make grid scroll by that amount
+    // adjust height of rows (extra pixel for grid lines)
+    // and make grid scroll by that amount
     dc.SetFont(GetDefaultCellFont());
-    int h = dc.GetCharHeight() + extraHeight;
+    int h = dc.GetCharHeight() + extraHeight + 1;
     SetRowMinimalAcceptableHeight(h);
     SetDefaultRowSize(h, true);
     SetScrollLineY(h);
 }
 //-----------------------------------------------------------------------------
-BEGIN_EVENT_TABLE(DataGrid, wxGrid)
-    EVT_CONTEXT_MENU(DataGrid::OnContextMenu)
-    EVT_GRID_CELL_RIGHT_CLICK(DataGrid::OnGridCellRightClick)
-    EVT_GRID_LABEL_RIGHT_CLICK(DataGrid::OnGridLabelRightClick)
-    EVT_MENU(DataGrid::ID_MENU_CANCELFETCHALL, DataGrid::OnMenuCancelFetchAll)
-    EVT_UPDATE_UI(DataGrid::ID_MENU_CANCELFETCHALL, DataGrid::OnMenuUpdateCancelFetchAll)
-    EVT_MENU(DataGrid::ID_MENU_CELLFONT, DataGrid::OnMenuCellFont)
-    EVT_MENU(DataGrid::ID_MENU_COPYTOCLIPBOARD, DataGrid::OnMenuCopyToCB)
-    EVT_UPDATE_UI(DataGrid::ID_MENU_COPYTOCLIPBOARD, DataGrid::OnMenuUpdateIfHasSelection)
-    EVT_MENU(DataGrid::ID_MENU_COPYTOCLIPBOARDASINSERT, DataGrid::OnMenuCopyToCBAsInsert)
-    EVT_UPDATE_UI(DataGrid::ID_MENU_COPYTOCLIPBOARDASINSERT, DataGrid::OnMenuUpdateIfHasSelection)
-    EVT_MENU(DataGrid::ID_MENU_COPYTOCLIPBOARDASUPDATE, DataGrid::OnMenuCopyToCBAsUpdate)
-    EVT_UPDATE_UI(DataGrid::ID_MENU_COPYTOCLIPBOARDASUPDATE, DataGrid::OnMenuUpdateIfHasSelection)
-    EVT_MENU(DataGrid::ID_MENU_FETCHALL, DataGrid::OnMenuFetchAll)
-    EVT_UPDATE_UI(DataGrid::ID_MENU_FETCHALL, DataGrid::OnMenuUpdateFetchAll)
-    EVT_MENU(DataGrid::ID_MENU_LABELFONT, DataGrid::OnMenuLabelFont)
-    EVT_MENU(DataGrid::ID_MENU_SAVEASHTML, DataGrid::OnMenuSaveAsHTML)
-    EVT_UPDATE_UI(DataGrid::ID_MENU_SAVEASHTML, DataGrid::OnMenuUpdateIfHasSelection)
-
-#ifdef __WXGTK__
-    EVT_MOUSEWHEEL(DataGrid::OnMouseWheel)
-    EVT_SCROLLWIN_THUMBRELEASE(DataGrid::OnThumbRelease)
-#endif
-END_EVENT_TABLE()
-//-----------------------------------------------------------------------------
-void DataGrid::OnContextMenu(wxContextMenuEvent& event)
-{
-    showPopMenu(event.GetPosition());
-}
-//-----------------------------------------------------------------------------
-void DataGrid::OnGridCellRightClick(wxGridEvent& event)
-{
-    showPopMenu(event.GetPosition());
-}
-//-----------------------------------------------------------------------------
-void DataGrid::OnGridLabelRightClick(wxGridEvent& WXUNUSED(event))
-{
-    showPopMenu(ScreenToClient(::wxGetMousePosition()));
-}
-//-----------------------------------------------------------------------------
-void DataGrid::OnIdle(wxIdleEvent& event)
-{
-    FR_TRY
-
-    DataGridTable* table = getDataGridTable();
-    // disconnect event handler if nothing more to be done, will be
-    // re-registered on next successfull execution of select statement
-    if (!table || !table->canFetchMoreRows())
-    {
-        Disconnect(wxID_ANY, wxEVT_IDLE);
-        return;
-    }
-    // fetch more rows until row cache is filled or timeslice is spent, and
-    // request another wxEVT_IDLE event if row cache has not been filled
-    if (table->needsMoreRowsFetched())
-    {
-        table->fetch();
-        if (table->needsMoreRowsFetched())
-            event.RequestMore();
-        AdjustScrollbars();
-    }
-
-    FR_CATCH
-}
-//-----------------------------------------------------------------------------
-void DataGrid::OnMenuCellFont(wxCommandEvent& WXUNUSED(event))
+void DataGrid::setCellFont()
 {
     wxFont f = ::wxGetFontFromUser(this, GetDefaultCellFont());
     if (f.Ok())
@@ -268,15 +249,14 @@ void DataGrid::OnMenuCellFont(wxCommandEvent& WXUNUSED(event))
     }
 }
 //-----------------------------------------------------------------------------
-void DataGrid::OnMenuCopyToCB(wxCommandEvent& WXUNUSED(event))
+void DataGrid::copyToCB()
 {
-    FR_TRY
-
     DataGridTable* table = getDataGridTable();
     if (!table)
         return;
 
     bool all = true;
+    bool any = false;
     {
         wxBusyCursor cr;
         wxString sRows;
@@ -292,6 +272,7 @@ void DataGrid::OnMenuCopyToCB(wxCommandEvent& WXUNUSED(event))
                     if (!sRow.IsEmpty())
                         sRow += wxT("\t");
                     sRow += table->getCellValue(i, j);
+                    any = true;
                 }
                 else
                     all = false;
@@ -303,27 +284,33 @@ void DataGrid::OnMenuCopyToCB(wxCommandEvent& WXUNUSED(event))
             copyToClipboard(sRows);
     }
 
+    if (!any)   // no cells selected -> copy a single cell
+    {
+        copyToClipboard(table->getCellValue(GetGridCursorRow(),
+            GetGridCursorCol()));
+    }
     if (all)
         notifyIfUnfetchedData();
-
-    FR_CATCH
 }
 //-----------------------------------------------------------------------------
-void DataGrid::OnMenuCopyToCBAsInsert(wxCommandEvent& WXUNUSED(event))
+void DataGrid::copyToCBAsInsert()
 {
-    wxBusyCursor cr;
-
     DataGridTable* table = getDataGridTable();
     if (!table)
         return;
+
+    bool all = true;
+    {
+        wxBusyCursor cr;
+
     // TODO: - using one table is not correct for JOINs or sub-SELECTs
-    //       -> should probably refuse to work if not from one table
+    //       - should probably refuse to work if not from one table
     //       - should probably refuse to create INSERT for "[...]"
+    //       - table&PK info is available in DataGridRows::statementTablesM
     wxString tableName = table->getTableName();
     // NOTE: this has been reworked (compared to myDataGrid), because
     //       not all rows have necessarily the same fields selected
     wxString sRows;
-    bool all = true;
     for (int i = 0; i < GetNumberRows(); i++)
     {
         wxString sCols;
@@ -356,24 +343,26 @@ void DataGrid::OnMenuCopyToCBAsInsert(wxCommandEvent& WXUNUSED(event))
     }
     if (!sRows.IsEmpty())
         copyToClipboard(sRows);
-
+    }
     if (all)
         notifyIfUnfetchedData();
 }
 //-----------------------------------------------------------------------------
-void DataGrid::OnMenuCopyToCBAsUpdate(wxCommandEvent& WXUNUSED(event))
+void DataGrid::copyToCBAsUpdate()
 {
-    wxBusyCursor cr;
-
     DataGridTable* table = getDataGridTable();
     if (!table)
         return;
+
+    bool all = true;
+    {
+        wxBusyCursor cr;
     // TODO: - using one table is not correct for JOINs or sub-SELECTs
-    //       -> should probably refuse to work if not from one table
+    //       - should probably refuse to work if not from one table
     //       - should probably refuse to create UPDATE for "[...]"
+    //       - we have this info in DataGridRows::statementTablesM
     wxString tableName = table->getTableName();
     wxString sRows;
-    bool all = true;
     for (int i = 0; i < GetNumberRows(); i++)
     {
         wxString str;
@@ -396,22 +385,11 @@ void DataGrid::OnMenuCopyToCBAsUpdate(wxCommandEvent& WXUNUSED(event))
             wxString where;
             // find primary key (otherwise use all values)
             Table *t = 0;
-            wxWindow* parent = GetParent();
-            while (parent)
+            Database* db = table->getDatabase();
+            if (db)
             {
-                if (dynamic_cast<ExecuteSqlFrame *>(parent))
-                    break;
-                parent = parent->GetParent();
-            }
-            ExecuteSqlFrame* frame = dynamic_cast<ExecuteSqlFrame *>(parent);
-            if (frame)
-            {
-                Database* db = frame->getDatabase();
-                if (db)
-                {
-                    t = dynamic_cast<Table *>(
-                        db->findByNameAndType(ntTable, tableName));
-                }
+                t = dynamic_cast<Table *>(
+                    db->findByNameAndType(ntTable, tableName));
             }
             if (!t)
             {
@@ -459,12 +437,12 @@ void DataGrid::OnMenuCopyToCBAsUpdate(wxCommandEvent& WXUNUSED(event))
     }
     if (!sRows.IsEmpty())
         copyToClipboard(sRows);
-
+    }
     if (all)
         notifyIfUnfetchedData();
 }
 //-----------------------------------------------------------------------------
-void DataGrid::OnMenuLabelFont(wxCommandEvent& WXUNUSED(event))
+void DataGrid::setHeaderFont()
 {
     wxFont f = ::wxGetFontFromUser(this, GetLabelFont());
     if (f.Ok())
@@ -478,12 +456,112 @@ void DataGrid::OnMenuLabelFont(wxCommandEvent& WXUNUSED(event))
     }
 }
 //-----------------------------------------------------------------------------
-void DataGrid::OnMenuSaveAsHTML(wxCommandEvent& WXUNUSED(event))
+void DataGrid::saveAsCSV()
+{
+    DataGridTable* table = getDataGridTable();
+    if (!table)
+        return;
+
+    wxString fname = ::wxFileSelector(_("Save data in selected cells as"),
+        wxEmptyString, wxEmptyString, wxT("*.csv"),
+        _("CSV files (*.csv)|*.csv|All files (*.*)|*.*"),
+#if wxCHECK_VERSION(2, 8, 0)
+        wxFD_SAVE | wxFD_CHANGE_DIR | wxFD_OVERWRITE_PROMPT, this);
+#else
+        wxSAVE | wxCHANGE_DIR | wxOVERWRITE_PROMPT, this);
+#endif
+    if (fname.empty())
+        return;
+
+    bool all = true;
+    {
+        wxBusyCursor cr;
+        // find all columns that have at least one cell selected
+        std::vector<bool> selCols;
+        int cols = GetNumberCols();
+        selCols.reserve(cols);
+        for (int j = 0; j < cols; j++)
+            selCols.push_back(false);
+
+        int rows = GetNumberRows();
+        for (int i = 0; i < rows; i++)
+        {
+            for (int j = 0; j < cols; j++)
+            {
+                if (IsInSelection(i, j))
+                    selCols[j] = true;
+            }
+        }
+
+        const wxString sCOMMA(wxT(","));
+        const wxString sTAB(wxT("\t"));
+        const wxString sAPS(wxT("\""));
+        const wxString sEOL(wxT("\n"));
+        wxString sDLM = sTAB;
+
+        bool dlmiscomma = false;
+        config().getValue(wxT("CSVDelimiterIsComma"), dlmiscomma);
+        if (dlmiscomma)
+            sDLM = sCOMMA;
+
+        // write CSV file
+        wxFileOutputStream fos(fname);
+        if (!fos.Ok()) // TODO: report error
+            return;
+        wxTextOutputStream outStr(fos);
+
+        wxString sHeader;
+        for (int j = 0; j < cols; j++)
+        {
+            if (selCols[j])
+            {
+                if (!sHeader.IsEmpty())
+                    sHeader += sDLM;
+                if (sDLM == sCOMMA)
+                    sHeader += sAPS + GetColLabelValue(j) + sAPS;
+                else
+                    sHeader += GetColLabelValue(j);
+            }
+        }
+        if (!sHeader.IsEmpty())
+            outStr.WriteString(sHeader + sEOL);
+
+        for (int i = 0; i < rows; i++)
+        {
+            wxString sRow;
+            for (int j = 0; j < cols; j++)
+            {
+                if (selCols[j])
+                {
+                    if (!sRow.IsEmpty())
+                        sRow += sDLM;
+                    if (sDLM == sCOMMA)
+                        sRow += table->getCellValueForCSV(i, j);
+                    else
+                        sRow += GetCellValue(i, j);
+                }
+                else
+                    all = false;
+            }
+            if (!sRow.IsEmpty())
+                outStr.WriteString(sRow + sEOL); // wxTextBuffer::GetEOL();
+        }
+    }
+
+    if (all)
+        notifyIfUnfetchedData();
+}
+//-----------------------------------------------------------------------------
+void DataGrid::saveAsHTML()
 {
     wxString fname = ::wxFileSelector(_("Save data in selected cells as"),
         wxEmptyString, wxEmptyString, wxT("*.html"),
         _("HTML files (*.html)|*.html|All files (*.*)|*.*"),
+#if wxCHECK_VERSION(2, 8, 0)
+        wxFD_SAVE | wxFD_CHANGE_DIR | wxFD_OVERWRITE_PROMPT, this);
+#else
         wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT, this);
+#endif
     if (fname.empty())
         return;
 
@@ -512,8 +590,9 @@ void DataGrid::OnMenuSaveAsHTML(wxCommandEvent& WXUNUSED(event))
 
     outStr.WriteString(wxT(
         "<html><head><META \
-            HTTP-EQUIV=\"Content-Type\" content=\"text/html; charset=utf-8\">\
-        </head>\
+            HTTP-EQUIV=\"Content-Type\" content=\"text/html; charset="));
+    outStr.WriteString(getHtmlCharset());
+    outStr.WriteString(wxT("\"></head>\
         <body bgcolor=white>\
         <table bgcolor=black cellspacing=1 cellpadding=3 border=0>\
             <tr>\n"));
@@ -574,37 +653,135 @@ void DataGrid::OnMenuSaveAsHTML(wxCommandEvent& WXUNUSED(event))
     outStr.WriteString(wxT("</table></body></html>\n"));
 }
 //-----------------------------------------------------------------------------
-void DataGrid::OnMenuUpdateIfHasSelection(wxUpdateUIEvent& event)
-{
-    event.Enable(IsSelection());
-}
-//-----------------------------------------------------------------------------
-void DataGrid::OnMenuCancelFetchAll(wxCommandEvent& WXUNUSED(event))
-{
-    DataGridTable* table = getDataGridTable();
-    if (table)
-        table->setFetchAllRecords(false);
-}
-//-----------------------------------------------------------------------------
-void DataGrid::OnMenuUpdateCancelFetchAll(wxUpdateUIEvent& event)
-{
-    DataGridTable* table = getDataGridTable();
-    event.Enable(table && table->canFetchMoreRows()
-        && table->getFetchAllRows());
-}
-//-----------------------------------------------------------------------------
-void DataGrid::OnMenuFetchAll(wxCommandEvent& WXUNUSED(event))
+void DataGrid::fetchAll()
 {
     DataGridTable* table = getDataGridTable();
     if (table)
         table->setFetchAllRecords(true);
 }
 //-----------------------------------------------------------------------------
-void DataGrid::OnMenuUpdateFetchAll(wxUpdateUIEvent& event)
+void DataGrid::cancelFetchAll()
 {
     DataGridTable* table = getDataGridTable();
-    event.Enable(table && table->canFetchMoreRows()
-        && !table->getFetchAllRows());
+    if (table)
+        table->setFetchAllRecords(false);
+}
+//-----------------------------------------------------------------------------
+BEGIN_EVENT_TABLE(DataGrid, wxGrid)
+    EVT_CONTEXT_MENU(DataGrid::OnContextMenu)
+    EVT_GRID_CELL_RIGHT_CLICK(DataGrid::OnGridCellRightClick)
+    EVT_GRID_LABEL_RIGHT_CLICK(DataGrid::OnGridLabelRightClick)
+    EVT_GRID_EDITOR_CREATED(DataGrid::OnEditorCreated)
+    //  EVT_GRID_EDITOR_HIDDEN( DataGrid::OnEditorHidden )
+    EVT_KEY_DOWN(DataGrid::OnKeyDown)
+#ifdef __WXGTK__
+    EVT_MOUSEWHEEL(DataGrid::OnMouseWheel)
+    EVT_SCROLLWIN_THUMBRELEASE(DataGrid::OnThumbRelease)
+#endif
+END_EVENT_TABLE()
+//-----------------------------------------------------------------------------
+void DataGrid::OnEditorCreated(wxGridEditorCreatedEvent& event)
+{
+    wxTextCtrl *editor = dynamic_cast<wxTextCtrl *>(event.GetControl());
+    if (!editor)
+        return;
+
+    editor->Connect(wxEVT_KEY_DOWN,
+        wxKeyEventHandler(DataGrid::OnEditorKeyDown),
+        0, this);
+}
+//-----------------------------------------------------------------------------
+void DataGrid::OnEditorKeyDown(wxKeyEvent& event)
+{
+    if (event.GetKeyCode() == WXK_DELETE)
+    {
+        wxTextCtrl *editor = dynamic_cast<wxTextCtrl *>(
+            event.GetEventObject());
+        DataGridTable *table = getDataGridTable();
+        if (editor && table && editor->GetValue().IsEmpty())
+        {
+            table->setNullFlag(true);
+            editor->SetValue(wxT("[null]"));
+            editor->SetSelection(-1,-1);
+            return; // prevent control from deleting [null]
+        }
+    }
+    event.Skip();
+}
+//-----------------------------------------------------------------------------
+void DataGrid::OnContextMenu(wxContextMenuEvent& WXUNUSED(event))
+{
+    // this doesn't work properly when cell editor is active
+    //  showPopMenu(event.GetPosition());
+    showPopMenu(ScreenToClient(::wxGetMousePosition()));
+}
+//-----------------------------------------------------------------------------
+void DataGrid::OnGridCellRightClick(wxGridEvent& event)
+{
+    showPopMenu(event.GetPosition());
+}
+//-----------------------------------------------------------------------------
+void DataGrid::OnGridLabelRightClick(wxGridEvent& WXUNUSED(event))
+{
+    showPopMenu(ScreenToClient(::wxGetMousePosition()));
+}
+//-----------------------------------------------------------------------------
+/*
+void DataGrid::OnGridSelectCell(wxGridEvent& event)
+{
+    DataGridTable* table = getDataGridTable();
+    if (table)
+        table->saveEditorChanges(event.GetRow());
+    event.Skip();
+}*/
+//-----------------------------------------------------------------------------
+void DataGrid::OnIdle(wxIdleEvent& event)
+{
+    DataGridTable* table = getDataGridTable();
+    // disconnect event handler if nothing more to be done, will be
+    // re-registered on next successfull execution of select statement
+    if (!table || !table->canFetchMoreRows())
+    {
+        Disconnect(wxID_ANY, wxEVT_IDLE);
+        return;
+    }
+    // fetch more rows until row cache is filled or timeslice is spent, and
+    // request another wxEVT_IDLE event if row cache has not been filled
+    if (table->needsMoreRowsFetched())
+    {
+        table->fetch();
+        if (table->needsMoreRowsFetched())
+            event.RequestMore();
+        AdjustScrollbars();
+    }
+}
+//-----------------------------------------------------------------------------
+void DataGrid::OnKeyDown(wxKeyEvent& event)
+{
+    if (event.GetKeyCode() == WXK_SPACE)
+    {
+#if wxCHECK_VERSION(2, 8, 0)
+        if (event.GetModifiers() == wxMOD_CONTROL)
+#else
+        if (event.ControlDown() && !event.AltDown() && !event.ShiftDown()
+            && !event.MetaDown())
+#endif
+        {
+            extendSelection(wxVERTICAL);
+            return;
+        }
+#if wxCHECK_VERSION(2, 8, 0)
+        if (event.GetModifiers() == wxMOD_SHIFT)
+#else
+        if (event.ShiftDown() && !event.AltDown() && !event.ControlDown()
+            && !event.MetaDown())
+#endif
+        {
+            extendSelection(wxHORIZONTAL);
+            return;
+        }
+    }
+    event.Skip();
 }
 //-----------------------------------------------------------------------------
 void DataGrid::OnMouseWheel(wxMouseEvent& event)
