@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////////
 //
-//	File    : $Id: statement.cpp 1011 2006-04-11 10:36:58Z mghie $
+//	File    : $Id: statement.cpp 1583 2007-12-09 19:57:48Z babuskov $
 //	Subject : IBPP, Service class implementation
 //
 ///////////////////////////////////////////////////////////////////////////////
@@ -75,7 +75,8 @@ void StatementImpl::Prepare(const std::string& sql)
 	// So we prefer to get them a little bit larger than needed than the other way.
 	int16_t inEstimate = 0;
 	int16_t outEstimate = 1;
-	for (size_t i = 0; i < strlen(sql.c_str()); i++)
+	size_t len = strlen(sql.c_str());
+	for (size_t i = 0; i < len; i++)
 	{
 		if (sql[i] == '?') ++inEstimate;
 		if (sql[i] == ',') ++outEstimate;
@@ -433,6 +434,9 @@ bool StatementImpl::Fetch()
 			_("isc_dsql_fetch failed."));
 	}
 
+    // Close the 'implicit' cursor to allow for forther Execute() calls
+    // on the prepared statement without fetching up to the last row
+	mCursorOpened = true;
 	return true;
 }
 
@@ -1275,15 +1279,13 @@ void StatementImpl::CursorFree()
 	}
 }
 
-StatementImpl::StatementImpl(DatabaseImpl* database, TransactionImpl* transaction,
-	const std::string& sql)
+StatementImpl::StatementImpl(DatabaseImpl* database, TransactionImpl* transaction)
 	: mRefCount(0), mHandle(0), mDatabase(0), mTransaction(0),
 	mInRow(0), mOutRow(0),
 	mResultSetAvailable(false), mCursorOpened(false), mType(IBPP::stUnknown)
 {
 	AttachDatabaseImpl(database);
 	if (transaction != 0) AttachTransactionImpl(transaction);
-	if (! sql.empty()) Prepare(sql);
 }
 
 StatementImpl::~StatementImpl()
