@@ -1,27 +1,27 @@
 /*
-Copyright (c) 2004, 2005, 2006 The FlameRobin Development Team
+  Copyright (c) 2004-2008 The FlameRobin Development Team
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+  Permission is hereby granted, free of charge, to any person obtaining
+  a copy of this software and associated documentation files (the
+  "Software"), to deal in the Software without restriction, including
+  without limitation the rights to use, copy, modify, merge, publish,
+  distribute, sublicense, and/or sell copies of the Software, and to
+  permit persons to whom the Software is furnished to do so, subject to
+  the following conditions:
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+  The above copyright notice and this permission notice shall be included
+  in all copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: CreateDDLVisitor.cpp 1228 2006-10-02 17:23:56Z mbabuskov $
+  $Id: CreateDDLVisitor.cpp 1631 2008-02-02 22:25:30Z mapopa $
 
 */
 //-----------------------------------------------------------------------------
@@ -39,7 +39,18 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
     #include "wx/wx.h"
 #endif
 
+#include <vector>
+
+#include "metadata/column.h"
+#include "metadata/constraints.h"
 #include "metadata/CreateDDLVisitor.h"
+#include "metadata/database.h"
+#include "metadata/domain.h"
+#include "metadata/generator.h"
+#include "metadata/Index.h"
+#include "metadata/role.h"
+#include "metadata/table.h"
+#include "metadata/view.h"
 
 // forward declaration to keep compilers happy
 void addIndex(std::vector<Index> *ix, wxString& sql, ColumnConstraint *cc);
@@ -72,15 +83,18 @@ wxString CreateDDLVisitor::getSuffixSql() const
 // this one is not called from "outside", but from visit(Table) function
 void CreateDDLVisitor::visitColumn(Column& c)
 {
-    preSqlM << c.getQuotedName() << wxT(" ");
-
     wxString computed = c.getComputedSource();
     if (!computed.IsEmpty())
     {
-        preSqlM << wxT("COMPUTED BY ") << computed;
+        wxString add = wxT("ALTER TABLE ") + c.getTable()->getQuotedName()
+            + wxT(" ADD ") + c.getQuotedName() + wxT(" COMPUTED BY ")
+            + computed + wxT(";\n");
+        postSqlM << add;
+        sqlM << add;
         return;
     }
 
+    preSqlM << c.getQuotedName() << wxT(" ");
     wxString collate = c.getCollation();
     Domain *d = c.getDomain();
     if (d)
@@ -89,7 +103,7 @@ void CreateDDLVisitor::visitColumn(Column& c)
         {
             preSqlM << d->getDatatypeAsString();
             wxString charset = d->getCharset();
-            Database *db = d->getDatabase();
+            Database* db = d->findDatabase();
             if (!charset.IsEmpty())
             {
                 if (!db || db->getDatabaseCharset() != charset)
@@ -159,16 +173,11 @@ void iterateit(CreateDDLVisitor* v, Database& db, ProgressIndicator* pi)
 // build the sql script for entire database
 void CreateDDLVisitor::visitDatabase(Database& d)
 {
-    // TODO: use progressIndicatorM to show what's going on, and check for
-    //       isCanceled()
     if (progressIndicatorM)
         progressIndicatorM->initProgress(wxEmptyString, 10, 0, 1);
 
     try
     {
-        preSqlM << _("/* Please note that this script doesn't maintain\n")
-                << _("   proper order of creation for computed columns if\n")
-                << _("   they reference other tables.  */\n\n");
         preSqlM << wxT("/********************* ROLES **********************/\n\n");
         iterateit<Role>(this, d, progressIndicatorM);
 
@@ -189,6 +198,7 @@ void CreateDDLVisitor::visitDatabase(Database& d)
 
         preSqlM << wxT("/********************* VIEWS **********************/\n\n");
         // TODO: build dependecy tree first, and order views by it
+        //       also include computed columns of tables?
         iterateit<View>(this, d, progressIndicatorM);
 
         preSqlM << wxT("/******************* EXCEPTIONS *******************/\n\n");
@@ -218,7 +228,7 @@ void CreateDDLVisitor::visitDomain(Domain& d)
     preSqlM += wxT("CREATE DOMAIN ") + d.getQuotedName() + wxT("\n AS ") +
             d.getDatatypeAsString();
     wxString charset = d.getCharset();
-    Database *db = d.getDatabase();
+    Database* db = d.findDatabase();
     if (!charset.IsEmpty() && (!db || db->getDatabaseCharset() != charset))
         preSqlM += wxT(" CHARACTER SET ") + charset;
     preSqlM += wxT("\n");
@@ -414,11 +424,6 @@ void CreateDDLVisitor::visitProcedure(Procedure& p)
     preSqlM << temp << wxT("\n");
 }
 //-----------------------------------------------------------------------------
-void CreateDDLVisitor::visitParameter(Parameter&)
-{
-    // empty
-}
-//-----------------------------------------------------------------------------
 void CreateDDLVisitor::visitRole(Role& r)
 {
     preSqlM += wxT("CREATE ROLE ") + r.getQuotedName() + wxT(";\n");
@@ -446,16 +451,6 @@ void CreateDDLVisitor::visitRole(Role& r)
     sqlM = preSqlM + wxT("\n") + postSqlM;
 }
 //-----------------------------------------------------------------------------
-void CreateDDLVisitor::visitRoot(Root&)
-{
-    // empty
-}
-//-----------------------------------------------------------------------------
-void CreateDDLVisitor::visitServer(Server&)
-{
-    // empty
-}
-//-----------------------------------------------------------------------------
 // used by visit(Table)
 void addIndex(std::vector<Index> *ix, wxString& sql, ColumnConstraint *cc)
 {
@@ -476,11 +471,18 @@ void addIndex(std::vector<Index> *ix, wxString& sql, ColumnConstraint *cc)
 //-----------------------------------------------------------------------------
 void CreateDDLVisitor::visitTable(Table& t)
 {
-    preSqlM += wxT("CREATE TABLE ") + t.getQuotedName() + wxT("\n(\n  ");
+    preSqlM += wxT("CREATE TABLE ") + t.getQuotedName();
+    wxString external = t.getExternalPath();
+    if (!external.IsEmpty())
+    {
+        external.Replace(wxT("'"), wxT("''"));
+        preSqlM += wxT(" EXTERNAL '") + external + wxT("'");
+    }
+    preSqlM += wxT("\n(\n  ");
     t.checkAndLoadColumns();
     for (MetadataCollection<Column>::iterator it=t.begin(); it!=t.end(); ++it)
     {
-        if (it != t.begin())
+        if (it != t.begin() && (*it).getComputedSource().IsEmpty())
             preSqlM += wxT(",\n  ");
         visitColumn(*it);
     }
@@ -529,17 +531,24 @@ void CreateDDLVisitor::visitTable(Table& t)
                 postSqlM += wxT("UNIQUE ");
             if ((*ci).getIndexType() == Index::itDescending)
                 postSqlM += wxT("DESCENDING ");
-            postSqlM += wxT("INDEX ") + (*ci).getQuotedName() + wxT(" ON ")
-                + t.getQuotedName() + wxT(" (");
-            std::vector<wxString> *cols = (*ci).getSegments();
-            for (std::vector<wxString>::const_iterator it = cols->begin(); it != cols->end(); ++it)
+            postSqlM += wxT("INDEX ") + (*ci).getQuotedName() + wxT(" ON ") + t.getQuotedName();
+            wxString expre = (*ci).getExpression();
+            if (!expre.IsEmpty())
+                postSqlM += wxT(" COMPUTED BY ") + expre;
+            else
             {
-                if (it != cols->begin())
-                    postSqlM += wxT(",");
-                Identifier id(*it);
-                postSqlM += id.getQuoted();
+                postSqlM += wxT(" (");
+                std::vector<wxString> *cols = (*ci).getSegments();
+                for (std::vector<wxString>::const_iterator it = cols->begin(); it != cols->end(); ++it)
+                {
+                    if (it != cols->begin())
+                        postSqlM += wxT(",");
+                    Identifier id(*it);
+                    postSqlM += id.getQuoted();
+                }
+                postSqlM += wxT(")");
             }
-            postSqlM += wxT(");\n");
+            postSqlM += wxT(";\n");
         }
     }
 
@@ -567,12 +576,12 @@ void CreateDDLVisitor::visitTable(Table& t)
 //-----------------------------------------------------------------------------
 void CreateDDLVisitor::visitTrigger(Trigger& t)
 {
-    wxString object, source, type, relation;
+    wxString object, type;
     bool active;
     int position;
     t.getTriggerInfo(object, active, position, type);
-    t.getSource(source);
-    t.getRelation(relation);
+    wxString source = t.getSource();
+    wxString relation = t.getRelation();
 
     preSqlM << wxT("SET TERM ^ ;\nCREATE TRIGGER ") << t.getQuotedName()
          << wxT(" FOR ") << relation;
@@ -669,8 +678,3 @@ void CreateDDLVisitor::visitView(View& v)
     sqlM += preSqlM + wxT("\n") + postSqlM;
 }
 //-----------------------------------------------------------------------------
-void CreateDDLVisitor::visitMetadataItem(MetadataItem&)
-{
-    // empty
-}
-//-----------------------------------------------------------------------------
