@@ -1,27 +1,27 @@
 /*
-Copyright (c) 2004, 2005, 2006 The FlameRobin Development Team
+  Copyright (c) 2004-2008 The FlameRobin Development Team
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+  Permission is hereby granted, free of charge, to any person obtaining
+  a copy of this software and associated documentation files (the
+  "Software"), to deal in the Software without restriction, including
+  without limitation the rights to use, copy, modify, merge, publish,
+  distribute, sublicense, and/or sell copies of the Software, and to
+  permit persons to whom the Software is furnished to do so, subject to
+  the following conditions:
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+  The above copyright notice and this permission notice shall be included
+  in all copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: database.cpp 1249 2006-11-17 16:06:49Z mbabuskov $
+  $Id: database.cpp 1631 2008-02-02 22:25:30Z mapopa $
 
 */
 
@@ -42,16 +42,16 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "config/Config.h"
 #include "config/DatabaseConfig.h"
+#include "core/FRError.h"
 #include "core/StringUtils.h"
-#include "dberror.h"
+#include "engine/MetadataLoader.h"
 #include "MasterPassword.h"
 #include "metadata/database.h"
 #include "metadata/MetadataItemVisitor.h"
 #include "metadata/parameter.h"
 #include "metadata/root.h"
-#include "sql/SimpleParser.h"
-#include "sql/SqlTokenizer.h"
 #include "sql/SqlStatement.h"
+#include "sql/SqlTokenizer.h"
 //-----------------------------------------------------------------------------
 using namespace std;
 //-----------------------------------------------------------------------------
@@ -150,11 +150,6 @@ bool DatabaseInfo::getReadOnly()
     return readOnlyM;
 }
 //-----------------------------------------------------------------------------
-int DatabaseInfo::getSize()
-{
-    return ((pagesM * pageSizeM) / 1024);
-}
-//-----------------------------------------------------------------------------
 int DatabaseInfo::getSweep()
 {
     return sweepM;
@@ -163,16 +158,15 @@ int DatabaseInfo::getSweep()
 void DatabaseInfo::loadInfo(const IBPP::Database* database)
 {
     (*database)->Info(&odsM, &odsMinorM, &pageSizeM, &pagesM,
-        &buffersM, &sweepM, &forcedWritesM, &reserveM);
+        &buffersM, &sweepM, &forcedWritesM, &reserveM, &readOnlyM);
     dialectM = (*database)->Dialect();
 }
 //-----------------------------------------------------------------------------
 Database::Database()
-    : MetadataItem(), idM(0), storeEncryptedPasswordM(false)
+    : MetadataItem(), metadataLoaderM(0), connectedM(false), connectionCredentialsM(0),
+    storeEncryptedPasswordM(false), idM(0)
 {
     typeM = ntDatabase;
-    connectedM = false;
-    connectionCredentialsM = 0;
 
     // has to be here, since notify() might be called before initChildren()
     domainsM.setProperties(this, wxT("Domains"), ntDomains);
@@ -190,18 +184,19 @@ Database::Database()
 }
 //-----------------------------------------------------------------------------
 Database::Database(const Database& rhs)
-    : MetadataItem(rhs), databaseM(rhs.databaseM), connectedM(rhs.connectedM),
-    databaseCharsetM(rhs.databaseCharsetM), pathM(rhs.pathM),
-    credentialsM(rhs.credentialsM), connectionCredentialsM(0),
-    domainsM(rhs.domainsM), exceptionsM(rhs.exceptionsM),
-    functionsM(rhs.functionsM), generatorsM(rhs.generatorsM),
-    proceduresM(rhs.proceduresM), rolesM(rhs.rolesM), tablesM(rhs.tablesM),
-    sysTablesM(rhs.sysTablesM), triggersM(rhs.triggersM),
-    viewsM(rhs.viewsM), collationsM(rhs.collationsM), idM(rhs.idM),
-    storeEncryptedPasswordM(rhs.storeEncryptedPasswordM)
+    : MetadataItem(rhs), databaseM(rhs.databaseM),
+        metadataLoaderM(0), connectedM(rhs.connectedM),
+        databaseCharsetM(rhs.databaseCharsetM), pathM(rhs.pathM),
+        credentialsM(rhs.credentialsM), connectionCredentialsM(0),
+        domainsM(rhs.domainsM), exceptionsM(rhs.exceptionsM),
+        functionsM(rhs.functionsM), generatorsM(rhs.generatorsM),
+        proceduresM(rhs.proceduresM), rolesM(rhs.rolesM), tablesM(rhs.tablesM),
+        sysTablesM(rhs.sysTablesM), triggersM(rhs.triggersM),
+        viewsM(rhs.viewsM), collationsM(rhs.collationsM), idM(rhs.idM), 
+        storeEncryptedPasswordM(rhs.storeEncryptedPasswordM)
 {
     if (rhs.connectionCredentialsM)
-        connectionCredentialsM = new Credentials(*connectionCredentialsM);
+        connectionCredentialsM = new Credentials(*rhs.connectionCredentialsM);
 
     domainsM.setParent(this);
     exceptionsM.setParent(this);
@@ -215,6 +210,11 @@ Database::Database(const Database& rhs)
     viewsM.setParent(this);
 }
 //-----------------------------------------------------------------------------
+Database::~Database()
+{
+    resetCredentials();
+}
+//-----------------------------------------------------------------------------
 void Database::prepareTemporaryCredentials()
 {
     resetCredentials();
@@ -286,35 +286,21 @@ wxString getLoadingSql(NodeType type)
 // This could be moved to Column class
 wxString Database::loadDomainNameForColumn(wxString table, wxString field)
 {
-    try
-    {
-        IBPP::Transaction tr1 = IBPP::TransactionFactory(databaseM, IBPP::amRead);
-        tr1->Start();
-        IBPP::Statement st1 = IBPP::StatementFactory(databaseM, tr1);
-        st1->Prepare(
-            "select rdb$field_source from rdb$relation_fields where rdb$relation_name = ? and rdb$field_name = ?"
-        );
-        st1->Set(1, wx2std(table));
-        st1->Set(2, wx2std(field));
-        st1->Execute();
-        st1->Fetch();
-        std::string domain;
-        st1->Get(1, domain);
-        tr1->Commit();
-        domain.erase(domain.find_last_not_of(" ") + 1);
-        return std2wx(domain);
-    }
-    catch (IBPP::Exception &e)
-    {
-        lastError().setMessage(std2wx(e.ErrorMessage()));
-    }
-    catch (...)
-    {
-        lastError().setMessage(_("System error."));
-    }
+    MetadataLoader* loader = getMetadataLoader();
+    MetadataLoaderTransaction tr(loader);
 
-    ::wxMessageBox(lastError().getMessage(), _("Postprocessing error."), wxOK|wxICON_WARNING);
-    return wxT("");
+    IBPP::Statement& st1 = loader->getStatement(
+        "select rdb$field_source from rdb$relation_fields"
+        " where rdb$relation_name = ? and rdb$field_name = ?"
+    );
+    st1->Set(1, wx2std(table));
+    st1->Set(2, wx2std(field));
+    st1->Execute();
+    st1->Fetch();
+    std::string domain;
+    st1->Get(1, domain);
+    domain.erase(domain.find_last_not_of(" ") + 1);
+    return std2wx(domain);
 }
 //-----------------------------------------------------------------------------
 //! returns all collations for a given charset
@@ -330,75 +316,50 @@ std::vector<wxString> Database::getCollations(wxString charset)
     return temp;
 }
 //-----------------------------------------------------------------------------
-Domain *Database::loadMissingDomain(wxString name)
+Domain* Database::loadMissingDomain(wxString name)
 {
-    try
+    MetadataLoader* loader = getMetadataLoader();
+    MetadataLoaderTransaction tr(loader);
+
+    IBPP::Statement& st1 = loader->getStatement(
+        "select count(*) from rdb$fields f"
+        " left outer join rdb$types t on f.rdb$field_type=t.rdb$type"
+        " where t.rdb$field_name='RDB$FIELD_TYPE' and f.rdb$field_name = ?"
+    );
+    st1->Set(1, wx2std(name));
+    st1->Execute();
+    if (st1->Fetch())
     {
-        IBPP::Transaction tr1 = IBPP::TransactionFactory(databaseM, IBPP::amRead);
-        tr1->Start();
-        IBPP::Statement st1 = IBPP::StatementFactory(databaseM, tr1);
-        st1->Prepare(
-            "select count(*) from rdb$fields f left outer join rdb$types t on f.rdb$field_type=t.rdb$type "
-            "where t.rdb$field_name='RDB$FIELD_TYPE' and f.rdb$field_name = ?"
-        );
-        st1->Set(1, wx2std(name));
-        st1->Execute();
-        if (st1->Fetch())
+        int c;
+        st1->Get(1, c);
+        if (c > 0)
         {
-            int c;
-            st1->Get(1, c);
-            if (c > 0)
-            {
-                Domain* d = domainsM.add(name); // add domain to collection
-                d->setParent(this);
-                if (name.substr(0, 4) != wxT("RDB$"))
-                    refreshByType(ntDomain);
-                return d;
-            }
+            Domain* d = domainsM.add(name); // add domain to collection
+            d->setParent(this);
+            if (name.substr(0, 4) != wxT("RDB$"))
+                refreshByType(ntDomain);
+            return d;
         }
-        tr1->Commit();
-    }
-    catch (IBPP::Exception &e)
-    {
-        lastError().setMessage(std2wx(e.ErrorMessage()));
-    }
-    catch (...)
-    {
-        lastError().setMessage(_("System error."));
     }
     return 0;
 }
 //-----------------------------------------------------------------------------
 //! small helper function, reads sql and fills the vector with values
 // this can be made template function in future
-bool Database::fillVector(std::vector<wxString>& list, wxString sql)
+void Database::fillVector(std::vector<wxString>& list, wxString sql)
 {
-    try
+    MetadataLoader* loader = getMetadataLoader();
+    MetadataLoaderTransaction tr(loader);
+
+    IBPP::Statement& st1 = loader->getStatement(wx2std(sql));
+    st1->Execute();
+    while (st1->Fetch())
     {
-        IBPP::Transaction tr1 = IBPP::TransactionFactory(databaseM, IBPP::amRead);
-        tr1->Start();
-        IBPP::Statement st1 = IBPP::StatementFactory(databaseM, tr1);
-        st1->Prepare(wx2std(sql));
-        st1->Execute();
-        while (st1->Fetch())
-        {
-            std::string s;
-            st1->Get(1, s);
-            s.erase(s.find_last_not_of(" ") + 1); // trim
-            list.push_back(std2wx(s));
-        }
-        tr1->Commit();
-        return true;
+        std::string s;
+        st1->Get(1, s);
+        s.erase(s.find_last_not_of(" ") + 1); // trim
+        list.push_back(std2wx(s));
     }
-    catch (IBPP::Exception &e)
-    {
-        lastError().setMessage(std2wx(e.ErrorMessage()));
-    }
-    catch (...)
-    {
-        lastError().setMessage(_("System error."));
-    }
-    return false;
 }
 //-----------------------------------------------------------------------------
 bool Database::isDefaultCollation(const wxString& charset,
@@ -417,76 +378,51 @@ void Database::loadCollations()
     if (!collationsM.empty())
         return;
 
-    try
-    {
-        IBPP::Transaction tr1 = IBPP::TransactionFactory(databaseM, IBPP::amRead);
-        tr1->Start();
-        IBPP::Statement st1 = IBPP::StatementFactory(databaseM, tr1);
-        st1->Prepare("select c.rdb$character_set_name, k.rdb$collation_name "
-            " from rdb$character_sets c"
-            " left outer join rdb$collations k "
-            "   on c.rdb$character_set_id = k.rdb$character_set_id "
-            " order by c.rdb$character_set_name, k.rdb$collation_id");
-        st1->Execute();
-        while (st1->Fetch())
-        {
-            std::string charset, collation;
-            st1->Get(1, charset);
-            st1->Get(2, collation);
-            charset.erase(charset.find_last_not_of(" ") + 1);
-            collation.erase(collation.find_last_not_of(" ") + 1);
-            collationsM.insert(std::multimap<wxString, wxString>::value_type(std2wx(charset), std2wx(collation)));
-        }
-        tr1->Commit();
-        return;
-    }
-    catch (IBPP::Exception &e)
-    {
-        lastError().setMessage(std2wx(e.ErrorMessage()));
-    }
-    catch (...)
-    {
-        lastError().setMessage(_("System error."));
-    }
+    MetadataLoader* loader = getMetadataLoader();
+    MetadataLoaderTransaction tr(loader);
 
-    ::wxMessageBox(lastError().getMessage(), _("Error while loading collations."), wxOK|wxICON_WARNING);
+    IBPP::Statement& st1 = loader->getStatement(
+        "select c.rdb$character_set_name, k.rdb$collation_name "
+        " from rdb$character_sets c"
+        " left outer join rdb$collations k "
+        "   on c.rdb$character_set_id = k.rdb$character_set_id "
+        " order by c.rdb$character_set_name, k.rdb$collation_id");
+    st1->Execute();
+    while (st1->Fetch())
+    {
+        std::string charset, collation;
+        st1->Get(1, charset);
+        st1->Get(2, collation);
+        charset.erase(charset.find_last_not_of(" ") + 1);
+        collation.erase(collation.find_last_not_of(" ") + 1);
+        collationsM.insert(std::multimap<wxString, wxString>::value_type(
+            std2wx(charset), std2wx(collation)));
+    }
 }
 //-----------------------------------------------------------------------------
 wxString Database::getTableForIndex(wxString indexName)
 {
-    try
-    {
-        IBPP::Transaction tr1 = IBPP::TransactionFactory(databaseM, IBPP::amRead);
-        tr1->Start();
-        IBPP::Statement st1 = IBPP::StatementFactory(databaseM, tr1);
-        st1->Prepare("SELECT rdb$relation_name from rdb$indices where rdb$index_name = ?");
-        st1->Set(1, wx2std(indexName));
-        st1->Execute();
-        if (st1->Fetch())
-        {
-            std::string retval;
-            st1->Get(1, retval);
-            retval.erase(retval.find_last_not_of(" ") + 1);
-            return std2wx(retval);
-        }
-    }
-    catch (IBPP::Exception &e)
-    {
-        lastError().setMessage(std2wx(e.ErrorMessage()));
-    }
-    catch (...)
-    {
-        lastError().setMessage(_("System error."));
-    }
+    MetadataLoader* loader = getMetadataLoader();
+    MetadataLoaderTransaction tr(loader);
 
-    ::wxMessageBox(lastError().getMessage(),
-        _("Error while loading table for index."), wxOK | wxICON_WARNING);
-    return wxT("");
+    IBPP::Statement& st1 = loader->getStatement(
+        "SELECT rdb$relation_name from rdb$indices where rdb$index_name = ?");
+    st1->Set(1, wx2std(indexName));
+    st1->Execute();
+
+    wxString tableName;
+    if (st1->Fetch())
+    {
+        std::string retval;
+        st1->Get(1, retval);
+        retval.erase(retval.find_last_not_of(" ") + 1);
+        tableName = std2wx(retval);
+    }
+    return tableName;
 }
 //-----------------------------------------------------------------------------
 //! load list of objects of type "type" from database, and fill the DBH
-bool Database::loadObjects(NodeType type, IBPP::Transaction& tr1,
-    ProgressIndicator* indicator)
+void Database::loadObjects(NodeType type, ProgressIndicator* indicator)
 {
     switch (type)
     {
@@ -500,50 +436,48 @@ bool Database::loadObjects(NodeType type, IBPP::Transaction& tr1,
         case ntFunction:    functionsM.clear();     break;
         case ntDomain:      domainsM.clear();       break;
         case ntException:   exceptionsM.clear();    break;
-        default:            return false;
+        default:            return;
     };
 
-    SubjectLocker locker(this);
-    try
-    {
-        IBPP::Statement st1 = IBPP::StatementFactory(databaseM, tr1);
-        st1->Prepare(wx2std(getLoadingSql(type)));
-        st1->Execute();
-        while (st1->Fetch())
-        {
-            std::string name;
-            st1->Get(1, name);
-            name.erase(name.find_last_not_of(" ") + 1);
-            addObject(type, std2wx(name));
+    MetadataLoader* loader = getMetadataLoader();
+    // first start a transaction for metadata loading, then lock the database
+    // when objects go out of scope and are destroyed, database will be
+    // unlocked before the transaction is committed - any update() calls on
+    // observers can possibly use the same transaction
+    MetadataLoaderTransaction tr(loader);
+    SubjectLocker lock(this);
 
-            if (indicator && indicator->isCanceled())
-                break;
-        }
-        refreshByType(type);
-        return true;
-    }
-    catch (IBPP::Exception &e)
+    IBPP::Statement& st1 = loader->getStatement(wx2std(getLoadingSql(type)));
+    st1->Execute();
+    while (st1->Fetch())
     {
-        lastError().setMessage(std2wx(e.ErrorMessage()));
-    }
-    catch (...)
-    {
-        lastError().setMessage(_("System error."));
-    }
+        std::string name;
+        st1->Get(1, name);
+        name.erase(name.find_last_not_of(" ") + 1);
+        addObject(type, std2wx(name));
 
-    return false;
+        if (indicator && indicator->isCanceled())
+            break;
+    }
+    refreshByType(type);
 }
 //-----------------------------------------------------------------------------
-bool Database::loadGeneratorValues()
+void Database::loadGeneratorValues()
 {
+    MetadataLoader* loader = getMetadataLoader();
+    // first start a transaction for metadata loading, then lock the database
+    // when objects go out of scope and are destroyed, database will be
+    // unlocked before the transaction is committed - any update() calls on
+    // observers can possibly use the same transaction
+    MetadataLoaderTransaction tr(loader);
+    SubjectLocker lock(this);
+
     for (MetadataCollection<Generator>::iterator it = generatorsM.begin();
         it != generatorsM.end(); ++it)
     {
-        if (!(*it).loadValue())
-            return false;
+        (*it).loadValue();
     }
     // generatorsM.notify() not necessary, loadValue() notifies
-    return true;
 }
 //-----------------------------------------------------------------------------
 //! Notify the observers that collection has changed
@@ -614,8 +548,10 @@ Relation* Database::findRelation(const Identifier& name)
 //-----------------------------------------------------------------------------
 Relation* Database::getRelationForTrigger(Trigger* trigger)
 {
-    wxString relName;
-    if (!trigger || !trigger->getRelation(relName))
+    if (!trigger)
+        return 0;
+    wxString relName = trigger->getRelation();
+    if (relName.IsEmpty())
         return 0;
     return findRelation(Identifier(relName));
 }
@@ -677,10 +613,10 @@ bool Database::addObject(NodeType type, wxString name)
 // alter table [name] alter [column] type [domain or datatype]
 // declare external function [name]
 // set null flag via system tables update
-bool Database::parseCommitedSql(const SqlStatement& stm)
+void Database::parseCommitedSql(const SqlStatement& stm)
 {
     if (!stm.isDDL())
-        return true;    // return false only on IBPP exception
+        return;    // return false only on IBPP exception
 
     // TODO: check that there are no unwanted side-effects to this
     // SubjectLocker locker(this);
@@ -690,7 +626,7 @@ bool Database::parseCommitedSql(const SqlStatement& stm)
         MetadataItem *obj = stm.getObject();
         if (obj)
             obj->notifyObservers();
-        return true;
+        return;
     }
 
     if (stm.actionIs(actDROP, ntIndex))
@@ -699,7 +635,7 @@ bool Database::parseCommitedSql(const SqlStatement& stm)
         MetadataCollection<Table>::iterator it;
         for (it = tablesM.begin(); it != tablesM.end(); ++it)
             (*it).invalidateIndices(stm.getName());
-        return true;
+        return;
     }
 
     // handle "CREATE INDEX", "ALTER INDEX" and "SET STATISTICS INDEX"
@@ -710,7 +646,7 @@ bool Database::parseCommitedSql(const SqlStatement& stm)
         MetadataItem* m = findByNameAndType(ntTable, tableName);
         if (Table* t = dynamic_cast<Table*>(m))
             t->invalidateIndices();
-        return true;
+        return;
     }
 
     // update all TABLEs and VIEWs on "DROP TRIGGER"
@@ -732,19 +668,19 @@ bool Database::parseCommitedSql(const SqlStatement& stm)
         Relation *r = stm.getCreateTriggerRelation();
         if (r)
             r->notifyObservers();
-        return true;
+        return;
     }
 
     MetadataItem *object = stm.getObject();
     if (!object)
-        return true;
+        return;
 
     if (stm.actionIs(actSET, ntGenerator) ||
         stm.actionIs(actALTER, ntGenerator))
     {
         if (Generator* g = dynamic_cast<Generator*>(object))
-            g->loadValue(true);
-        return true;
+            g->loadValue();
+        return;
     }
 
     if (stm.actionIs(actDROP))
@@ -765,7 +701,7 @@ bool Database::parseCommitedSql(const SqlStatement& stm)
                     it++;
             }
         }
-        return true;
+        return;
     }
 
     if (stm.isAlterColumn())
@@ -797,22 +733,22 @@ bool Database::parseCommitedSql(const SqlStatement& stm)
     {
         object->getDescription(true);   // force reload
         object->notifyObservers();
-        return true;
+        return;
     }
 
     if (stm.actionIs(actALTER))
     {
         // TODO: this is a place where we would simply call virtual invalidate() function
         // and object would do wherever it needs to
-        bool success = true;
         switch (stm.getObjectType())
         {
             case ntTable:
             case ntView:
-                success = dynamic_cast<Relation*>(object)->loadColumns();
+                dynamic_cast<Relation*>(object)->loadColumns();
                 break;
             case ntProcedure:
-                success = dynamic_cast<Procedure*>(object)->checkAndLoadParameters(true); // force reload
+                dynamic_cast<Procedure*>(object)->checkAndLoadParameters(true); // force reload
+                dynamic_cast<Procedure*>(object)->checkDependentProcedures();
                 break;
             case ntException:
                 dynamic_cast<Exception*>(object)->loadProperties(true);
@@ -833,71 +769,42 @@ bool Database::parseCommitedSql(const SqlStatement& stm)
                     break;
                 }
             case ntDomain:
-                if (!dynamic_cast<Domain *>(object)->loadInfo())
-                    success = false;
-                else    // notify all table columns with that domain
-                {
-                    for (MetadataCollection<Table>::iterator it = tablesM.begin(); it != tablesM.end(); ++it)
-                        for (MetadataCollection<Column>::iterator i2 = (*it).begin(); i2 != (*it).end(); ++i2)
-                            if ((*i2).getSource() == stm.getName())
-                                (*i2).notifyObservers();
-                }
+                dynamic_cast<Domain *>(object)->loadInfo();
+                // notify all table columns with that domain
+                for (MetadataCollection<Table>::iterator it = tablesM.begin(); it != tablesM.end(); ++it)
+                    for (MetadataCollection<Column>::iterator i2 = (*it).begin(); i2 != (*it).end(); ++i2)
+                        if ((*i2).getSource() == stm.getName())
+                            (*i2).notifyObservers();
                 break;
             default:
                 object->notifyObservers();
                 break;
         }
-        if (!success)
-            return false;
     }
-    return true;
 }
 //-----------------------------------------------------------------------------
-bool Database::drop()
+void Database::drop()
 {
-    try
-    {
-        databaseM->Drop();
-        return true;
-    }
-    catch (IBPP::Exception &e)
-    {
-        lastError().setMessage(std2wx(e.ErrorMessage()));
-    }
-    catch (...)
-    {
-        lastError().setMessage(_("System error."));
-    }
-    return false;
+    databaseM->Drop();
 }
 //-----------------------------------------------------------------------------
-bool Database::reconnect() const
+void Database::reconnect()
 {
-    try
-    {
-        databaseM->Disconnect();
-        databaseM->Connect();
+    // must recreate, because IBPP::Database member will become invalid
+    delete metadataLoaderM;
+    metadataLoaderM = 0;
 
-        return true;
-    }
-    catch (IBPP::Exception &e)
-    {
-        lastError().setMessage(std2wx(e.ErrorMessage()));
-    }
-    catch (...)
-    {
-        lastError().setMessage(_("System error."));
-    }
-    return false;
+    databaseM->Disconnect();
+    databaseM->Connect();
 }
 //-----------------------------------------------------------------------------
 // the caller of this function should check whether the database object has the
 // password set, and if it does not, it should provide the password
 //               and if it does, just provide that password
-bool Database::connect(wxString password, ProgressIndicator* indicator)
+void Database::connect(wxString password, ProgressIndicator* indicator)
 {
     if (connectedM)
-        return true;
+        return;
 
     try
     {
@@ -913,15 +820,20 @@ bool Database::connect(wxString password, ProgressIndicator* indicator)
         tablesM.setParent(this);
         sysTablesM.setParent(this);
 
+        // first start a transaction for metadata loading, then lock the
+        // database
+        // when objects go out of scope and are destroyed, database will be
+        // unlocked before the transaction is committed - any update() calls
+        // on observers can possibly use the same transaction
+        MetadataLoaderTransaction tr(getMetadataLoader());
+        SubjectLocker lock(this);
+
         bool canceled = (indicator && indicator->isCanceled());
         if (!canceled)
         {
-            IBPP::Transaction tr1 = IBPP::TransactionFactory(databaseM, IBPP::amRead);
-            tr1->Start();
-
             // load database charset
-            IBPP::Statement st1 = IBPP::StatementFactory(databaseM, tr1);
-            st1->Prepare("select rdb$character_set_name from rdb$database");
+            IBPP::Statement& st1 = metadataLoaderM->getStatement(
+                "select rdb$character_set_name from rdb$database");
             st1->Execute();
             if (st1->Fetch())
             {
@@ -931,28 +843,34 @@ bool Database::connect(wxString password, ProgressIndicator* indicator)
             }
 
             // load metadata information
+#if wxCHECK_VERSION(2, 9, 0)
+            struct NodeTypeName { NodeType type; const char* name; };
+#else
             struct NodeTypeName { NodeType type; const wxChar* name; };
+#endif
             static const NodeTypeName nodetypes[] = {
-                { ntTable, _("Tables") }, { ntSysTable, _("System tables") },
-                { ntView, _("Views") }, { ntProcedure, _("Procedures") },
-                { ntTrigger, _("Triggers") }, { ntRole, _("Roles") },
-                { ntDomain, _("Domains") }, { ntFunction, _("Functions") },
-                { ntGenerator, _("Generators") }, { ntException, _("Exceptions")  }
+                { ntTable, wxTRANSLATE("Tables") },
+                { ntSysTable, wxTRANSLATE("System tables") },
+                { ntView, wxTRANSLATE("Views") },
+                { ntProcedure, wxTRANSLATE("Procedures") },
+                { ntTrigger, wxTRANSLATE("Triggers") },
+                { ntRole, wxTRANSLATE("Roles") },
+                { ntDomain, wxTRANSLATE("Domains") },
+                { ntFunction, wxTRANSLATE("Functions") },
+                { ntGenerator, wxTRANSLATE("Generators") },
+                { ntException, wxTRANSLATE("Exceptions") }
             };
 
             int typeCount = sizeof(nodetypes) / sizeof(NodeTypeName);
             for (int i = 0; i < typeCount; i++)
             {
-                // Only load system tables when they should be shown
-                if (nodetypes[i].type == ntSysTable && !showSysTables())
-                    continue;
-
                 if (indicator)
                 {
+                    wxString typeName(wxGetTranslation(nodetypes[i].name));
                     indicator->initProgress(wxString::Format(_("Loading %s..."),
-                        nodetypes[i].name), typeCount, i);
+                        typeName.c_str()), typeCount, i);
                 }
-                loadObjects(nodetypes[i].type, tr1, indicator);
+                loadObjects(nodetypes[i].type, indicator);
                 if (indicator && indicator->isCanceled())
                 {
                     canceled = true;
@@ -961,82 +879,68 @@ bool Database::connect(wxString password, ProgressIndicator* indicator)
             }
             if (!canceled && indicator)
                 indicator->initProgress(_("Complete"), typeCount, typeCount);
-            tr1->Commit();
         }
         if (canceled)
             disconnect();
 
         if (connectedM)
             databaseInfoM.loadInfo(&databaseM);
-
-        return true;
-    }
-    catch (IBPP::Exception &e)
-    {
-        lastError().setMessage(std2wx(e.ErrorMessage()));
     }
     catch (...)
     {
-        lastError().setMessage(_("System error."));
+        try
+        {
+            disconnect();
+            databaseM.clear();
+        }
+        catch (...) // we don't care as we already have an error to report
+        {
+        }
+        throw;
     }
-
-    disconnect();
-    databaseM.clear();
-    return false;
 }
 //-----------------------------------------------------------------------------
-bool Database::disconnect(bool onlyDBH)
+void Database::disconnect(bool onlyDBH)
 {
     if (!connectedM && !onlyDBH)
-        return true;
+        return;
 
-    try
-    {
-        if (!onlyDBH)
-            databaseM->Disconnect();
-        resetCredentials();     // "forget" temporary username/password
-        connectedM = false;
+    if (!onlyDBH)
+        databaseM->Disconnect();
+    delete metadataLoaderM;
+    metadataLoaderM = 0;
+    resetCredentials();     // "forget" temporary username/password
+    connectedM = false;
 
-        // remove entire DBH beneath
-        domainsM.clear();
-        functionsM.clear();
-        generatorsM.clear();
-        proceduresM.clear();
-        rolesM.clear();
-        tablesM.clear();
-        triggersM.clear();
-        viewsM.clear();
-        exceptionsM.clear();
+    // remove entire DBH beneath
+    domainsM.clear();
+    functionsM.clear();
+    generatorsM.clear();
+    proceduresM.clear();
+    rolesM.clear();
+    sysTablesM.clear();
+    tablesM.clear();
+    triggersM.clear();
+    viewsM.clear();
+    exceptionsM.clear();
 
-        // this a special case for Database only since it doesn't destroy its subitems
-        // but only hides them (i.e. getChildren returns nothing, but items are present)
-        // so observers must get removed
-        domainsM.detachAllObservers();
-        functionsM.detachAllObservers();
-        generatorsM.detachAllObservers();
-        proceduresM.detachAllObservers();
-        rolesM.detachAllObservers();
-        tablesM.detachAllObservers();
-        sysTablesM.detachAllObservers();
-        triggersM.detachAllObservers();
-        viewsM.detachAllObservers();
-        exceptionsM.detachAllObservers();
+    // this a special case for Database only since it doesn't destroy its subitems
+    // but only hides them (i.e. getChildren returns nothing, but items are present)
+    // so observers must get removed
+    domainsM.detachAllObservers();
+    functionsM.detachAllObservers();
+    generatorsM.detachAllObservers();
+    proceduresM.detachAllObservers();
+    rolesM.detachAllObservers();
+    sysTablesM.detachAllObservers();
+    tablesM.detachAllObservers();
+    triggersM.detachAllObservers();
+    viewsM.detachAllObservers();
+    exceptionsM.detachAllObservers();
 
-        if (config().get(wxT("HideDisconnectedDatabases"), false))
-            getServer()->notifyObservers();
-        notifyObservers();
-        return true;
-    }
-    catch (IBPP::Exception &e)
-    {
-        lastError().setMessage(std2wx(e.ErrorMessage()));
-    }
-    catch (...)
-    {
-        lastError().setMessage(_("System error."));
-    }
-
-    return false;
+    if (config().get(wxT("HideDisconnectedDatabases"), false))
+        getServer()->notifyObservers();
+    notifyObservers();
 }
 //-----------------------------------------------------------------------------
 void Database::clear()
@@ -1053,6 +957,13 @@ bool Database::isConnected() const
     return connectedM;
 }
 //-----------------------------------------------------------------------------
+MetadataLoader* Database::getMetadataLoader()
+{
+    if (metadataLoaderM == 0)
+        metadataLoaderM = new MetadataLoader(*this, 8);
+    return metadataLoaderM;
+}
+//-----------------------------------------------------------------------------
 bool Database::getChildren(std::vector<MetadataItem*>& temp)
 {
     if (!connectedM)
@@ -1181,6 +1092,10 @@ wxString Database::getRawPassword() const
 //-----------------------------------------------------------------------------
 wxString Database::getDecryptedPassword() const
 {
+    // if we already have an established connection return that password
+    if (databaseM != 0 && databaseM->Connected())
+        return std2wx(databaseM->UserPassword());
+
     // temporary connection
     if (connectionCredentialsM)
         return connectionCredentialsM->getPassword();
@@ -1189,7 +1104,7 @@ wxString Database::getDecryptedPassword() const
         return wxEmptyString;
 
     if (storeEncryptedPasswordM)
-        return decryptPassword(raw, getUsername()+getConnectionString());
+        return decryptPassword(raw, getUsername() + getConnectionString());
     else
         return raw;
 }
